# -*- Mode: Python; py-indent-offset: 4 -*-
# test_generictreemodel - Tests for GenericTreeModel
# Copyright (C) 2013 Simon Feltman
#
#   test_generictreemodel.py: Tests for GenericTreeModel
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

# system
import gc
import sys
import weakref
import unittest
import platform

# pygobject
from gi.repository import GObject

try:
    from gi.repository import Gtk
    from pygtkcompat.generictreemodel import GenericTreeModel
    from pygtkcompat.generictreemodel import _get_user_data_as_pyobject
    has_gtk = True
except ImportError:
    GenericTreeModel = object
    has_gtk = False


class Node(object):
    """Represents a generic node with name, value, and children."""
    def __init__(self, name, value, *children):
        self.name = name
        self.value = value
        self.children = list(children)
        self.parent = None
        self.next = None

        for i, child in enumerate(children):
            child.parent = weakref.ref(self)
            if i < len(children) - 1:
                child.next = weakref.ref(children[i + 1])

    def __repr__(self):
        return 'Node("%s", %s)' % (self.name, self.value)


class ATesterModel(GenericTreeModel):
    def __init__(self):
        super(ATesterModel, self).__init__()
        self.root = Node('root', 0,
                         Node('spam', 1,
                              Node('sushi', 2),
                              Node('bread', 3)
                         ),
                         Node('eggs', 4)
                        )

    def on_get_flags(self):
        return 0

    def on_get_n_columns(self):
        return 2

    def on_get_column_type(self, n):
        return (str, int)[n]

    def on_get_iter(self, path):
        node = self.root
        path = list(path)
        idx = path.pop(0)
        while path:
            idx = path.pop(0)
            node = node.children[idx]
        return node

    def on_get_path(self, node):
        def rec_get_path(n):
            for i, child in enumerate(n.children):
                if child == node:
                    return [i]
                else:
                    res = rec_get_path(child)
                    if res:
                        res.insert(0, i)

        return rec_get_path(self.root)

    def on_get_value(self, node, column):
        if column == 0:
            return node.name
        elif column == 1:
            return node.value

    def on_iter_has_child(self, node):
        return bool(node.children)

    def on_iter_next(self, node):
        if node.next:
            return node.next()

    def on_iter_children(self, node):
        if node:
            return node.children[0]
        else:
            return self.root

    def on_iter_n_children(self, node):
        if node is None:
            return 1
        return len(node.children)

    def on_iter_nth_child(self, node, n):
        if node is None:
            assert n == 0
            return self.root
        return node.children[n]

    def on_iter_parent(self, child):
        if child.parent:
            return child.parent()


@unittest.skipUnless(has_gtk, 'Gtk not available')
class TestReferences(unittest.TestCase):
    def setUp(self):
        pass

    @unittest.skipIf(platform.python_implementation() == "PyPy", "not with PyPy")
    def test_c_tree_iter_user_data_as_pyobject(self):
        obj = object()
        obj_id = id(obj)
        ref_count = sys.getrefcount(obj)

        # This is essentially a stolen ref in the context of _CTreeIter.get_user_data_as_pyobject
        it = Gtk.TreeIter()
        it.user_data = obj_id

        obj2 = _get_user_data_as_pyobject(it)
        self.assertEqual(obj, obj2)
        self.assertEqual(sys.getrefcount(obj), ref_count + 1)

    def test_leak_references_on(self):
        model = ATesterModel()
        obj_ref = weakref.ref(model.root)
        # Initial refcount is 1 for model.root + the temporary
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 2)

        # Iter increases by 1 do to assignment to iter.user_data
        res, it = model.do_get_iter([0])
        self.assertEqual(id(model.root), it.user_data)
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 3)

        # Verify getting a TreeIter more then once does not further increase
        # the ref count.
        res2, it2 = model.do_get_iter([0])
        self.assertEqual(id(model.root), it2.user_data)
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 3)

        # Deleting the iter does not decrease refcount because references
        # leak by default (they are stored in the held_refs pool)
        del it
        gc.collect()
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 3)

        # Deleting a model should free all held references to user data
        # stored by TreeIters
        del model
        gc.collect()
        self.assertEqual(obj_ref(), None)

    def test_row_deleted_frees_refs(self):
        model = ATesterModel()
        obj_ref = weakref.ref(model.root)
        # Initial refcount is 1 for model.root + the temporary
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 2)

        # Iter increases by 1 do to assignment to iter.user_data
        res, it = model.do_get_iter([0])
        self.assertEqual(id(model.root), it.user_data)
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 3)

        # Notifying the underlying model of a row_deleted should decrease the
        # ref count.
        model.row_deleted(Gtk.TreePath('0'), model.root)
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 2)

        # Finally deleting the actual object should collect it completely
        del model.root
        gc.collect()
        self.assertEqual(obj_ref(), None)

    def test_leak_references_off(self):
        model = ATesterModel()
        model.leak_references = False

        obj_ref = weakref.ref(model.root)
        # Initial refcount is 1 for model.root + the temporary
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 2)

        # Iter does not increas count by 1 when leak_references is false
        res, it = model.do_get_iter([0])
        self.assertEqual(id(model.root), it.user_data)
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 2)

        # Deleting the iter does not decrease refcount because assigning user_data
        # eats references and does not release them.
        del it
        gc.collect()
        if hasattr(sys, "getrefcount"):
            self.assertEqual(sys.getrefcount(model.root), 2)

        # Deleting the model decreases the final ref, and the object is collected
        del model
        gc.collect()
        self.assertEqual(obj_ref(), None)

    def test_iteration_refs(self):
        # Pull iterators off the model using the wrapped C API which will
        # then call back into the python overrides.
        model = ATesterModel()
        nodes = [node for node in model.iter_depth_first()]
        values = [node.value for node in nodes]

        # Verify depth first ordering
        self.assertEqual(values, [0, 1, 2, 3, 4])

        # Verify ref counts for each of the nodes.
        # 5 refs for each node at this point:
        #   1 - ref held in getrefcount function
        #   2 - ref held by "node" var during iteration
        #   3 - ref held by local "nodes" var
        #   4 - ref held by the root/children graph itself
        #   5 - ref held by the model "held_refs" instance var
        for node in nodes:
            if hasattr(sys, "getrefcount"):
                self.assertEqual(sys.getrefcount(node), 5)

        # A second iteration and storage of the nodes in a new list
        # should only increase refcounts by 1 even though new
        # iterators are created and assigned.
        nodes2 = [node for node in model.iter_depth_first()]
        for node in nodes2:
            if hasattr(sys, "getrefcount"):
                self.assertEqual(sys.getrefcount(node), 6)

        # Hold weak refs and start verifying ref collection.
        node_refs = [weakref.ref(node) for node in nodes]

        # First round of collection
        del nodes2
        gc.collect()
        for node in nodes:
            if hasattr(sys, "getrefcount"):
                self.assertEqual(sys.getrefcount(node), 5)

        # Second round of collection, no more local lists of nodes.
        del nodes
        gc.collect()
        for ref in node_refs:
            node = ref()
            if hasattr(sys, "getrefcount"):
                self.assertEqual(sys.getrefcount(node), 4)

        # Using invalidate_iters or row_deleted(path, node) will clear out
        # the pooled refs held internal to the GenericTreeModel implementation.
        model.invalidate_iters()
        self.assertEqual(len(model._held_refs), 0)
        gc.collect()
        for ref in node_refs:
            node = ref()
            if hasattr(sys, "getrefcount"):
                self.assertEqual(sys.getrefcount(node), 3)

        # Deleting the root node at this point should allow all nodes to be collected
        # as there is no longer a way to reach the children
        del node  # node still in locals() from last iteration
        del model.root
        gc.collect()
        for ref in node_refs:
            self.assertEqual(ref(), None)


@unittest.skipUnless(has_gtk, 'Gtk not available')
class TestIteration(unittest.TestCase):
    def test_iter_next_root(self):
        model = ATesterModel()
        it = model.get_iter([0])
        self.assertEqual(it.user_data, id(model.root))
        self.assertEqual(model.root.next, None)

        it = model.iter_next(it)
        self.assertEqual(it, None)

    def test_iter_next_multiple(self):
        model = ATesterModel()
        it = model.get_iter([0, 0])
        self.assertEqual(it.user_data, id(model.root.children[0]))

        it = model.iter_next(it)
        self.assertEqual(it.user_data, id(model.root.children[1]))

        it = model.iter_next(it)
        self.assertEqual(it, None)


class ErrorModel(GenericTreeModel):
    # All on_* methods will raise a NotImplementedError by default
    pass


@unittest.skipUnless(has_gtk, 'Gtk not available')
class ExceptHook(object):
    """
    Temporarily installs an exception hook in a context which
    expects the given exc_type to be raised. This allows verification
    of exceptions that occur within python gi callbacks but
    are never bubbled through from python to C back to python.
    This works because exception hooks are called in PyErr_Print.
    """
    def __init__(self, *expected_exc_types):
        self._expected_exc_types = expected_exc_types
        self._exceptions = []

    def _excepthook(self, exc_type, value, traceback):
        self._exceptions.append((exc_type, value))

    def __enter__(self):
        self._oldhook = sys.excepthook
        sys.excepthook = self._excepthook
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        sys.excepthook = self._oldhook
        error_message = 'Expecting the following exceptions: %s, got: %s' % \
            (str(self._expected_exc_types), '\n'.join([str(item) for item in self._exceptions]))

        assert len(self._expected_exc_types) == len(self._exceptions), error_message

        for expected, got in zip(self._expected_exc_types, [exc[0] for exc in self._exceptions]):
            assert issubclass(got, expected), error_message


@unittest.skipUnless(has_gtk, 'Gtk not available')
class TestReturnsAfterError(unittest.TestCase):
    def setUp(self):
        self.model = ErrorModel()

    def test_get_flags(self):
        with ExceptHook(NotImplementedError):
            flags = self.model.get_flags()
        self.assertEqual(flags, 0)

    def test_get_n_columns(self):
        with ExceptHook(NotImplementedError):
            count = self.model.get_n_columns()
        self.assertEqual(count, 0)

    def test_get_column_type(self):
        with ExceptHook(NotImplementedError, ValueError):
            col_type = self.model.get_column_type(0)
        self.assertEqual(col_type, GObject.TYPE_INVALID)

    def test_get_iter(self):
        with ExceptHook(NotImplementedError):
            self.assertRaises(ValueError, self.model.get_iter, Gtk.TreePath(0))

    def test_get_path(self):
        it = self.model.create_tree_iter('foo')
        with ExceptHook(NotImplementedError):
            path = self.model.get_path(it)
        self.assertEqual(path, None)

    def test_get_value(self):
        it = self.model.create_tree_iter('foo')
        with ExceptHook(NotImplementedError):
            try:
                self.model.get_value(it, 0)
            except TypeError:
                pass  # silence TypeError converting None to GValue

    def test_iter_has_child(self):
        it = self.model.create_tree_iter('foo')
        with ExceptHook(NotImplementedError):
            res = self.model.iter_has_child(it)
        self.assertEqual(res, False)

    def test_iter_next(self):
        it = self.model.create_tree_iter('foo')
        with ExceptHook(NotImplementedError):
            res = self.model.iter_next(it)
        self.assertEqual(res, None)

    def test_iter_children(self):
        with ExceptHook(NotImplementedError):
            res = self.model.iter_children(None)
        self.assertEqual(res, None)

    def test_iter_n_children(self):
        with ExceptHook(NotImplementedError):
            res = self.model.iter_n_children(None)
        self.assertEqual(res, 0)

    def test_iter_nth_child(self):
        with ExceptHook(NotImplementedError):
            res = self.model.iter_nth_child(None, 0)
        self.assertEqual(res, None)

    def test_iter_parent(self):
        child = self.model.create_tree_iter('foo')
        with ExceptHook(NotImplementedError):
            res = self.model.iter_parent(child)
        self.assertEqual(res, None)



#!/usr/bin/env python
# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
#
# Copyright (C) 2014 Gian Mario Tagliaretti <gianmt@gnome.org>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

title = "FlowBox"
description = """
A FlowBox allows flexible and responsive grids which reflow as needed and
support sorting and filtering. The children of a GtkFlowBox are regular widgets.
"""

from gi.repository import Gtk, Gdk


class FlowBoxApp:
    def __init__(self):
        window = Gtk.Window()
        window.connect('destroy', lambda x: Gtk.main_quit())
        window.set_border_width(10)
        window.set_default_size(600, 400)

        header = Gtk.HeaderBar(title="Flow Box")
        header.set_subtitle("Sample FlowBox app")
        header.props.show_close_button = True

        window.set_titlebar(header)

        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)

        flowbox = Gtk.FlowBox()
        flowbox.set_valign(Gtk.Align.START)
        flowbox.set_max_children_per_line(30)
        flowbox.set_selection_mode(Gtk.SelectionMode.NONE)

        self.create_flowbox(flowbox)

        scrolled.add(flowbox)

        window.add(scrolled)
        window.show_all()

    def color_swatch_new(self, str_color):
        rgba = Gdk.RGBA()
        rgba.parse(str_color)

        text_rgba = Gdk.RGBA()  # default is white
        if max(rgba.red, rgba.green, rgba.blue) > 0.6:
            text_rgba.parse('black')

        label = Gtk.Label(label=str_color)
        label.override_background_color(0, rgba)
        label.override_color(0, text_rgba)
        return label

    def create_flowbox(self, flowbox):
        colors = [
            'AliceBlue',
            'AntiqueWhite',
            'AntiqueWhite1',
            'AntiqueWhite2',
            'AntiqueWhite3',
            'AntiqueWhite4',
            'aqua',
            'aquamarine',
            'aquamarine1',
            'aquamarine2',
            'aquamarine3',
            'aquamarine4',
            'azure',
            'azure1',
            'azure2',
            'azure3',
            'azure4',
            'beige',
            'bisque',
            'bisque1',
            'bisque2',
            'bisque3',
            'bisque4',
            'black',
            'BlanchedAlmond',
            'blue',
            'blue1',
            'blue2',
            'blue3',
            'blue4',
            'BlueViolet',
            'brown',
            'brown1',
            'brown2',
            'brown3',
            'brown4',
            'burlywood',
            'burlywood1',
            'burlywood2',
            'burlywood3',
            'burlywood4',
            'CadetBlue',
            'CadetBlue1',
            'CadetBlue2',
            'CadetBlue3',
            'CadetBlue4',
            'chartreuse',
            'chartreuse1',
            'chartreuse2',
            'chartreuse3',
            'chartreuse4',
            'chocolate',
            'chocolate1',
            'chocolate2',
            'chocolate3',
            'chocolate4',
            'coral',
            'coral1',
            'coral2',
            'coral3',
            'coral4',
            'CornflowerBlue',
            'cornsilk',
            'cornsilk1',
            'cornsilk2',
            'cornsilk3',
            'cornsilk4',
            'crimson',
            'cyan',
            'cyan1',
            'cyan2',
            'cyan3',
            'cyan4',
            'DarkBlue',
            'DarkCyan',
            'DarkGoldenrod',
            'DarkGoldenrod1',
            'DarkGoldenrod2',
            'DarkGoldenrod3',
            'DarkGoldenrod4',
            'DarkGray',
            'DarkGreen',
            'DarkGrey',
            'DarkKhaki',
            'DarkMagenta',
            'DarkOliveGreen',
            'DarkOliveGreen1',
            'DarkOliveGreen2',
            'DarkOliveGreen3',
            'DarkOliveGreen4',
            'DarkOrange',
            'DarkOrange1',
            'DarkOrange2',
            'DarkOrange3',
            'DarkOrange4',
            'DarkOrchid',
            'DarkOrchid1',
            'DarkOrchid2',
            'DarkOrchid3',
            'DarkOrchid4',
            'DarkRed',
            'DarkSalmon',
            'DarkSeaGreen',
            'DarkSeaGreen1',
            'DarkSeaGreen2',
            'DarkSeaGreen3',
            'DarkSeaGreen4',
            'DarkSlateBlue',
            'DarkSlateGray',
            'DarkSlateGray1',
            'DarkSlateGray2',
            'DarkSlateGray3',
            'DarkSlateGray4',
            'DarkSlateGrey',
            'DarkTurquoise',
            'DarkViolet',
            'DeepPink',
            'DeepPink1',
            'DeepPink2',
            'DeepPink3',
            'DeepPink4',
            'DeepSkyBlue',
            'DeepSkyBlue1',
            'DeepSkyBlue2',
            'DeepSkyBlue3',
            'DeepSkyBlue4',
            'DimGray',
            'DimGrey',
            'DodgerBlue',
            'DodgerBlue1',
            'DodgerBlue2',
            'DodgerBlue3',
            'DodgerBlue4',
            'firebrick',
            'firebrick1',
            'firebrick2',
            'firebrick3',
            'firebrick4',
            'FloralWhite',
            'ForestGreen',
            'fuchsia',
            'gainsboro',
            'GhostWhite',
            'gold',
            'gold1',
            'gold2',
            'gold3',
            'gold4',
            'goldenrod',
            'goldenrod1',
            'goldenrod2',
            'goldenrod3',
            'goldenrod4',
            'gray',
            'gray0',
            'gray1',
            'gray2',
            'gray3',
            'gray4',
            'gray5',
            'gray6',
            'gray7',
            'gray8',
            'gray9',
            'gray10',
            'gray11',
            'gray12',
            'gray13',
            'gray14',
            'gray15',
            'gray16',
            'gray17',
            'gray18',
            'gray19',
            'gray20',
            'gray21',
            'gray22',
            'gray23',
            'gray24',
            'gray25',
            'gray26',
            'gray27',
            'gray28',
            'gray29',
            'gray30',
            'gray31',
            'gray32',
            'gray33',
            'gray34',
            'gray35',
            'gray36',
            'gray37',
            'gray38',
            'gray39',
            'gray40',
            'gray41',
            'gray42',
            'gray43',
            'gray44',
            'gray45',
            'gray46',
            'gray47',
            'gray48',
            'gray49',
            'gray50',
            'gray51',
            'gray52',
            'gray53',
            'gray54',
            'gray55',
            'gray56',
            'gray57',
            'gray58',
            'gray59',
            'gray60',
            'gray61',
            'gray62',
            'gray63',
            'gray64',
            'gray65',
            'gray66',
            'gray67',
            'gray68',
            'gray69',
            'gray70',
            'gray71',
            'gray72',
            'gray73',
            'gray74',
            'gray75',
            'gray76',
            'gray77',
            'gray78',
            'gray79',
            'gray80',
            'gray81',
            'gray82',
            'gray83',
            'gray84',
            'gray85',
            'gray86',
            'gray87',
            'gray88',
            'gray89',
            'gray90',
            'gray91',
            'gray92',
            'gray93',
            'gray94',
            'gray95',
            'gray96',
            'gray97',
            'gray98',
            'gray99',
            'gray100',
            'green',
            'green1',
            'green2',
            'green3',
            'green4',
            'GreenYellow',
            'grey',
            'grey0',
            'grey1',
            'grey2',
            'grey3',
            'grey4',
            'grey5',
            'grey6',
            'grey7',
            'grey8',
            'grey9',
            'grey10',
            'grey11',
            'grey12',
            'grey13',
            'grey14',
            'grey15',
            'grey16',
            'grey17',
            'grey18',
            'grey19',
            'grey20',
            'grey21',
            'grey22',
            'grey23',
            'grey24',
            'grey25',
            'grey26',
            'grey27',
            'grey28',
            'grey29',
            'grey30',
            'grey31',
            'grey32',
            'grey33',
            'grey34',
            'grey35',
            'grey36',
            'grey37',
            'grey38',
            'grey39',
            'grey40',
            'grey41',
            'grey42',
            'grey43',
            'grey44',
            'grey45',
            'grey46',
            'grey47',
            'grey48',
            'grey49',
            'grey50',
            'grey51',
            'grey52',
            'grey53',
            'grey54',
            'grey55',
            'grey56',
            'grey57',
            'grey58',
            'grey59',
            'grey60',
            'grey61',
            'grey62',
            'grey63',
            'grey64',
            'grey65',
            'grey66',
            'grey67',
            'grey68',
            'grey69',
            'grey70',
            'grey71',
            'grey72',
            'grey73',
            'grey74',
            'grey75',
            'grey76',
            'grey77',
            'grey78',
            'grey79',
            'grey80',
            'grey81',
            'grey82',
            'grey83',
            'grey84',
            'grey85',
            'grey86',
            'grey87',
            'grey88',
            'grey89',
            'grey90',
            'grey91',
            'grey92',
            'grey93',
            'grey94',
            'grey95',
            'grey96',
            'grey97',
            'grey98',
            'grey99',
            'grey100',
            'honeydew',
            'honeydew1',
            'honeydew2',
            'honeydew3',
            'honeydew4',
            'HotPink',
            'HotPink1',
            'HotPink2',
            'HotPink3',
            'HotPink4',
            'IndianRed',
            'IndianRed1',
            'IndianRed2',
            'IndianRed3',
            'IndianRed4',
            'indigo',
            'ivory',
            'ivory1',
            'ivory2',
            'ivory3',
            'ivory4',
            'khaki',
            'khaki1',
            'khaki2',
            'khaki3',
            'khaki4',
            'lavender',
            'LavenderBlush',
            'LavenderBlush1',
            'LavenderBlush2',
            'LavenderBlush3',
            'LavenderBlush4',
            'LawnGreen',
            'LemonChiffon',
            'LemonChiffon1',
            'LemonChiffon2',
            'LemonChiffon3',
            'LemonChiffon4',
            'LightBlue',
            'LightBlue1',
            'LightBlue2',
            'LightBlue3',
            'LightBlue4',
            'LightCoral',
            'LightCyan',
            'LightCyan1',
            'LightCyan2',
            'LightCyan3',
            'LightCyan4',
            'LightGoldenrod',
            'LightGoldenrod1',
            'LightGoldenrod2',
            'LightGoldenrod3',
            'LightGoldenrod4',
            'LightGoldenrodYellow',
            'LightGray',
            'LightGreen',
            'LightGrey',
            'LightPink',
            'LightPink1',
            'LightPink2',
            'LightPink3',
            'LightPink4',
            'LightSalmon',
            'LightSalmon1',
            'LightSalmon2',
            'LightSalmon3',
            'LightSalmon4',
            'LightSeaGreen',
            'LightSkyBlue',
            'LightSkyBlue1',
            'LightSkyBlue2',
            'LightSkyBlue3',
            'LightSkyBlue4',
            'LightSlateBlue',
            'LightSlateGray',
            'LightSlateGrey',
            'LightSteelBlue',
            'LightSteelBlue1',
            'LightSteelBlue2',
            'LightSteelBlue3',
            'LightSteelBlue4',
            'LightYellow',
            'LightYellow1',
            'LightYellow2',
            'LightYellow3',
            'LightYellow4',
            'lime',
            'LimeGreen',
            'linen',
            'magenta',
            'magenta1',
            'magenta2',
            'magenta3',
            'magenta4',
            'maroon',
            'maroon1',
            'maroon2',
            'maroon3',
            'maroon4',
            'MediumAquamarine',
            'MediumBlue',
            'MediumOrchid',
            'MediumOrchid1',
            'MediumOrchid2',
            'MediumOrchid3',
            'MediumOrchid4',
            'MediumPurple',
            'MediumPurple1',
            'MediumPurple2',
            'MediumPurple3',
            'MediumPurple4',
            'MediumSeaGreen',
            'MediumSlateBlue',
            'MediumSpringGreen',
            'MediumTurquoise',
            'MediumVioletRed',
            'MidnightBlue',
            'MintCream',
            'MistyRose',
            'MistyRose1',
            'MistyRose2',
            'MistyRose3',
            'MistyRose4',
            'moccasin',
            'NavajoWhite',
            'NavajoWhite1',
            'NavajoWhite2',
            'NavajoWhite3',
            'NavajoWhite4',
            'navy',
            'NavyBlue',
            'OldLace',
            'olive',
            'OliveDrab',
            'OliveDrab1',
            'OliveDrab2',
            'OliveDrab3',
            'OliveDrab4',
            'orange',
            'orange1',
            'orange2',
            'orange3',
            'orange4',
            'OrangeRed',
            'OrangeRed1',
            'OrangeRed2',
            'OrangeRed3',
            'OrangeRed4',
            'orchid',
            'orchid1',
            'orchid2',
            'orchid3',
            'orchid4',
            'PaleGoldenrod',
            'PaleGreen',
            'PaleGreen1',
            'PaleGreen2',
            'PaleGreen3',
            'PaleGreen4',
            'PaleTurquoise',
            'PaleTurquoise1',
            'PaleTurquoise2',
            'PaleTurquoise3',
            'PaleTurquoise4',
            'PaleVioletRed',
            'PaleVioletRed1',
            'PaleVioletRed2',
            'PaleVioletRed3',
            'PaleVioletRed4',
            'PapayaWhip',
            'PeachPuff',
            'PeachPuff1',
            'PeachPuff2',
            'PeachPuff3',
            'PeachPuff4',
            'peru',
            'pink',
            'pink1',
            'pink2',
            'pink3',
            'pink4',
            'plum',
            'plum1',
            'plum2',
            'plum3',
            'plum4',
            'PowderBlue',
            'purple',
            'purple1',
            'purple2',
            'purple3',
            'purple4',
            'red',
            'red1',
            'red2',
            'red3',
            'red4',
            'RosyBrown',
            'RosyBrown1',
            'RosyBrown2',
            'RosyBrown3',
            'RosyBrown4',
            'RoyalBlue',
            'RoyalBlue1',
            'RoyalBlue2',
            'RoyalBlue3',
            'RoyalBlue4',
            'SaddleBrown',
            'salmon',
            'salmon1',
            'salmon2',
            'salmon3',
            'salmon4',
            'SandyBrown',
            'SeaGreen',
            'SeaGreen1',
            'SeaGreen2',
            'SeaGreen3',
            'SeaGreen4',
            'seashell',
            'seashell1',
            'seashell2',
            'seashell3',
            'seashell4',
            'sienna',
            'sienna1',
            'sienna2',
            'sienna3',
            'sienna4',
            'silver',
            'SkyBlue',
            'SkyBlue1',
            'SkyBlue2',
            'SkyBlue3',
            'SkyBlue4',
            'SlateBlue',
            'SlateBlue1',
            'SlateBlue2',
            'SlateBlue3',
            'SlateBlue4',
            'SlateGray',
            'SlateGray1',
            'SlateGray2',
            'SlateGray3',
            'SlateGray4',
            'SlateGrey',
            'snow',
            'snow1',
            'snow2',
            'snow3',
            'snow4',
            'SpringGreen',
            'SpringGreen1',
            'SpringGreen2',
            'SpringGreen3',
            'SpringGreen4',
            'SteelBlue',
            'SteelBlue1',
            'SteelBlue2',
            'SteelBlue3',
            'SteelBlue4',
            'tan',
            'tan1',
            'tan2',
            'tan3',
            'tan4',
            'teal',
            'thistle',
            'thistle1',
            'thistle2',
            'thistle3',
            'thistle4',
            'tomato',
            'tomato1',
            'tomato2',
            'tomato3',
            'tomato4',
            'turquoise',
            'turquoise1',
            'turquoise2',
            'turquoise3',
            'turquoise4',
            'violet',
            'VioletRed',
            'VioletRed1',
            'VioletRed2',
            'VioletRed3',
            'VioletRed4',
            'wheat',
            'wheat1',
            'wheat2',
            'wheat3',
            'wheat4',
            'white',
            'WhiteSmoke',
            'yellow',
            'yellow1',
            'yellow2',
            'yellow3',
            'yellow4',
            'YellowGreen',
        ]

        for color in colors:
            button = self.color_swatch_new(color)
            flowbox.add(button)


def main(demoapp=None):
    FlowBoxApp()
    Gtk.main()


if __name__ == '__main__':
    main()



# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, see <http://www.gnu.org/licenses/>.

import sys

PY2 = PY3 = False
if sys.version_info[0] == 2:
    PY2 = True

    from StringIO import StringIO
    StringIO

    from UserList import UserList
    UserList

    long_ = eval("long")
    integer_types = eval("(int, long)")
    string_types = eval("(basestring,)")
    text_type = eval("unicode")

    reload = eval("reload")
    xrange = eval("xrange")
    cmp = eval("cmp")

    exec("def reraise(tp, value, tb):\n raise tp, value, tb")
else:
    PY3 = True

    from io import StringIO
    StringIO

    from collections import UserList
    UserList

    long_ = int
    integer_types = (int,)
    string_types = (str,)
    text_type = str

    from importlib import reload
    reload
    xrange = range
    cmp = lambda a, b: (a > b) - (a < b)

    def reraise(tp, value, tb):
        raise tp(value).with_traceback(tb)



# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

from __future__ import absolute_import

import unittest

from gi.repository import GLib
from gi.repository import Gio


try:
    Gio.bus_get_sync(Gio.BusType.SESSION, None)
except GLib.Error:
    has_dbus = False
else:
    has_dbus = True


class TestDBusNodeInfo(unittest.TestCase):

    def test_new_for_xml(self):
        info = Gio.DBusNodeInfo.new_for_xml("""
<!DOCTYPE node PUBLIC '-//freedesktop//DTD D-BUS Object Introspection 1.0//EN'
    'http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd'>
<node>
    <interface name='org.freedesktop.DBus.Introspectable'>
        <method name='Introspect'>
            <arg name='data' direction='out' type='s'/>
        </method>
    </interface>
</node>
""")

        interfaces = info.interfaces
        del info
        assert len(interfaces) == 1
        assert interfaces[0].name == "org.freedesktop.DBus.Introspectable"
        methods = interfaces[0].methods
        del interfaces
        assert len(methods) == 1
        assert methods[0].name == "Introspect"
        out_args = methods[0].out_args
        assert len(out_args)
        del methods
        assert out_args[0].name == "data"


@unittest.skipUnless(has_dbus, "no dbus running")
class TestGDBusClient(unittest.TestCase):
    def setUp(self):
        self.bus = Gio.bus_get_sync(Gio.BusType.SESSION, None)

        self.dbus_proxy = Gio.DBusProxy.new_sync(self.bus,
                                                 Gio.DBusProxyFlags.NONE,
                                                 None,
                                                 'org.freedesktop.DBus',
                                                 '/org/freedesktop/DBus',
                                                 'org.freedesktop.DBus',
                                                 None)

    def test_native_calls_sync(self):
        result = self.dbus_proxy.call_sync('ListNames', None,
                                           Gio.DBusCallFlags.NO_AUTO_START,
                                           500, None)
        self.assertTrue(isinstance(result, GLib.Variant))
        result = result.unpack()[0]  # result is always a tuple
        self.assertTrue(len(result) > 1)
        self.assertTrue('org.freedesktop.DBus' in result)

        result = self.dbus_proxy.call_sync('GetNameOwner',
                                           GLib.Variant('(s)', ('org.freedesktop.DBus',)),
                                           Gio.DBusCallFlags.NO_AUTO_START, 500, None)
        self.assertTrue(isinstance(result, GLib.Variant))
        self.assertEqual(type(result.unpack()[0]), type(''))

    def test_native_calls_sync_errors(self):
        # error case: invalid argument types
        try:
            self.dbus_proxy.call_sync('GetConnectionUnixProcessID', None,
                                      Gio.DBusCallFlags.NO_AUTO_START, 500, None)
            self.fail('call with invalid arguments should raise an exception')
        except Exception as e:
            self.assertTrue('InvalidArgs' in str(e))

        # error case: invalid argument
        try:
            self.dbus_proxy.call_sync('GetConnectionUnixProcessID',
                                      GLib.Variant('(s)', (' unknown',)),
                                      Gio.DBusCallFlags.NO_AUTO_START, 500, None)
            self.fail('call with invalid arguments should raise an exception')
        except Exception as e:
            self.assertTrue('NameHasNoOwner' in str(e))

        # error case: unknown method
        try:
            self.dbus_proxy.call_sync('UnknownMethod', None,
                                      Gio.DBusCallFlags.NO_AUTO_START, 500, None)
            self.fail('call for unknown method should raise an exception')
        except Exception as e:
            self.assertTrue('UnknownMethod' in str(e))

    def test_native_calls_async(self):
        def call_done(obj, result, user_data):
            try:
                user_data['result'] = obj.call_finish(result)
            finally:
                user_data['main_loop'].quit()

        main_loop = GLib.MainLoop()
        data = {'main_loop': main_loop}
        self.dbus_proxy.call('ListNames', None,
                             Gio.DBusCallFlags.NO_AUTO_START, 500, None,
                             call_done, data)
        main_loop.run()

        self.assertTrue(isinstance(data['result'], GLib.Variant))
        result = data['result'].unpack()[0]  # result is always a tuple
        self.assertTrue(len(result) > 1)
        self.assertTrue('org.freedesktop.DBus' in result)

    def test_native_calls_async_errors(self):
        def call_done(obj, result, user_data):
            try:
                obj.call_finish(result)
                self.fail('call_finish() for unknown method should raise an exception')
            except Exception as e:
                self.assertTrue('UnknownMethod' in str(e))
            finally:
                user_data['main_loop'].quit()

        main_loop = GLib.MainLoop()
        data = {'main_loop': main_loop}
        self.dbus_proxy.call('UnknownMethod', None,
                             Gio.DBusCallFlags.NO_AUTO_START, 500, None,
                             call_done, data)
        main_loop.run()

    def test_python_calls_sync(self):
        # single value return tuples get unboxed to the one element
        result = self.dbus_proxy.ListNames('()')
        self.assertTrue(isinstance(result, list))
        self.assertTrue(len(result) > 1)
        self.assertTrue('org.freedesktop.DBus' in result)

        result = self.dbus_proxy.GetNameOwner('(s)', 'org.freedesktop.DBus')
        self.assertEqual(type(result), type(''))

        # empty return tuples get unboxed to None
        self.assertEqual(self.dbus_proxy.ReloadConfig('()'), None)

        # multiple return values remain a tuple; unfortunately D-BUS itself
        # does not have any method returning multiple results, so try talking
        # to notification-daemon (and don't fail the test if it does not exist)
        try:
            nd = Gio.DBusProxy.new_sync(self.bus,
                                        Gio.DBusProxyFlags.NONE, None,
                                        'org.freedesktop.Notifications',
                                        '/org/freedesktop/Notifications',
                                        'org.freedesktop.Notifications',
                                        None)
            result = nd.GetServerInformation('()')
            self.assertTrue(isinstance(result, tuple))
            self.assertEqual(len(result), 4)
            for i in result:
                self.assertEqual(type(i), type(''))
        except Exception as e:
            if 'Error.ServiceUnknown' not in str(e):
                raise

        # test keyword argument; timeout=0 will fail immediately
        try:
            self.dbus_proxy.GetConnectionUnixProcessID('(s)', '1', timeout=0)
            self.fail('call with timeout=0 should raise an exception')
        except Exception as e:
            # FIXME: this is not very precise, but in some environments we
            # do not always get an actual timeout
            self.assertTrue(isinstance(e, GLib.GError), str(e))

    def test_python_calls_sync_noargs(self):
        # methods without arguments don't need an explicit signature
        result = self.dbus_proxy.ListNames()
        self.assertTrue(isinstance(result, list))
        self.assertTrue(len(result) > 1)
        self.assertTrue('org.freedesktop.DBus' in result)

    def test_python_calls_sync_errors(self):
        # error case: invalid argument types
        try:
            self.dbus_proxy.GetConnectionUnixProcessID('()')
            self.fail('call with invalid arguments should raise an exception')
        except Exception as e:
            self.assertTrue('InvalidArgs' in str(e), str(e))

        try:
            self.dbus_proxy.GetConnectionUnixProcessID(None, 'foo')
            self.fail('call with None signature should raise an exception')
        except TypeError as e:
            self.assertTrue('signature' in str(e), str(e))

    def test_python_calls_async(self):
        def call_done(obj, result, user_data):
            user_data['result'] = result
            user_data['main_loop'].quit()

        main_loop = GLib.MainLoop()
        data = {'main_loop': main_loop}
        self.dbus_proxy.ListNames('()', result_handler=call_done, user_data=data)
        main_loop.run()

        result = data['result']
        self.assertEqual(type(result), type([]))
        self.assertTrue(len(result) > 1)
        self.assertTrue('org.freedesktop.DBus' in result)

    def test_python_calls_async_error_result(self):
        # when only specifying a result handler, this will get the error
        def call_done(obj, result, user_data):
            user_data['result'] = result
            user_data['main_loop'].quit()

        main_loop = GLib.MainLoop()
        data = {'main_loop': main_loop}
        self.dbus_proxy.ListNames('(s)', 'invalid_argument',
                                  result_handler=call_done, user_data=data)
        main_loop.run()

        self.assertTrue(isinstance(data['result'], Exception))
        self.assertTrue('InvalidArgs' in str(data['result']), str(data['result']))

    def test_python_calls_async_error(self):
        # when specifying an explicit error handler, this will get the error
        def call_done(obj, result, user_data):
            user_data['main_loop'].quit()
            self.fail('result handler should not be called')

        def call_error(obj, error, user_data):
            user_data['error'] = error
            user_data['main_loop'].quit()

        main_loop = GLib.MainLoop()
        data = {'main_loop': main_loop}
        self.dbus_proxy.ListNames('(s)', 'invalid_argument',
                                  result_handler=call_done,
                                  error_handler=call_error,
                                  user_data=data)
        main_loop.run()

        self.assertTrue(isinstance(data['error'], Exception))
        self.assertTrue('InvalidArgs' in str(data['error']), str(data['error']))

    def test_instantiate_custom_proxy(self):
        class SomeProxy(Gio.DBusProxy):
            def __init__(self):
                Gio.DBusProxy.__init__(self)

        SomeProxy()



#!/usr/bin/env python
# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
#
# Copyright (C) 2010 Red Hat, Inc., John (J5) Palmieri <johnp@redhat.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

title = "List Store"
description = """
The GtkListStore is used to store data in list form, to be used later on by a
GtkTreeView to display it. This demo builds a simple GtkListStore and displays
it. See the Stock Browser demo for a more advanced example.
"""


from gi.repository import Gtk, GObject, GLib


class Bug:
    def __init__(self, is_fixed, number, severity, description):
        self.is_fixed = is_fixed
        self.number = number
        self.severity = severity
        self.description = description


# initial data we use to fill in the store
data = [Bug(False, 60482, "Normal", "scrollable notebooks and hidden tabs"),
        Bug(False, 60620, "Critical", "gdk_window_clear_area (gdkwindow-win32.c) is not thread-safe"),
        Bug(False, 50214, "Major", "Xft support does not clean up correctly"),
        Bug(True, 52877, "Major", "GtkFileSelection needs a refresh method. "),
        Bug(False, 56070, "Normal", "Can't click button after setting in sensitive"),
        Bug(True, 56355, "Normal", "GtkLabel - Not all changes propagate correctly"),
        Bug(False, 50055, "Normal", "Rework width/height computations for TreeView"),
        Bug(False, 58278, "Normal", "gtk_dialog_set_response_sensitive () doesn't work"),
        Bug(False, 55767, "Normal", "Getters for all setters"),
        Bug(False, 56925, "Normal", "Gtkcalender size"),
        Bug(False, 56221, "Normal", "Selectable label needs right-click copy menu"),
        Bug(True, 50939, "Normal", "Add shift clicking to GtkTextView"),
        Bug(False, 6112, "Enhancement", "netscape-like collapsable toolbars"),
        Bug(False, 1, "Normal", "First bug :=)")]


class ListStoreApp:
    (COLUMN_FIXED,
     COLUMN_NUMBER,
     COLUMN_SEVERITY,
     COLUMN_DESCRIPTION,
     COLUMN_PULSE,
     COLUMN_ICON,
     COLUMN_ACTIVE,
     COLUMN_SENSITIVE,
     NUM_COLUMNS) = range(9)

    def __init__(self):
        self.window = Gtk.Window()
        self.window.set_title('Gtk.ListStore Demo')
        self.window.connect('destroy', Gtk.main_quit)

        vbox = Gtk.VBox(spacing=8)
        self.window.add(vbox)

        label = Gtk.Label(label='This is the bug list (note: not based on real data, it would be nice to have a nice ODBC interface to bugzilla or so, though).')
        vbox.pack_start(label, False, False, 0)

        sw = Gtk.ScrolledWindow()
        sw.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
        sw.set_policy(Gtk.PolicyType.NEVER,
                      Gtk.PolicyType.AUTOMATIC)
        vbox.pack_start(sw, True, True, 0)

        self.create_model()
        treeview = Gtk.TreeView(model=self.model)
        treeview.set_rules_hint(True)
        treeview.set_search_column(self.COLUMN_DESCRIPTION)
        sw.add(treeview)

        self.add_columns(treeview)

        self.window.set_default_size(280, 250)
        self.window.show_all()

        self.window.connect('delete-event', self.window_closed)
        self.timeout = GLib.timeout_add(80, self.spinner_timeout)

    def window_closed(self, window, event):
        if self.timeout != 0:
            GLib.source_remove(self.timeout)

    def spinner_timeout(self):
        if self.model is None:
            return False

        iter_ = self.model.get_iter_first()
        pulse = self.model.get(iter_, self.COLUMN_PULSE)[0]
        if pulse == 999999999:
            pulse = 0
        else:
            pulse += 1

        self.model.set_value(iter_, self.COLUMN_PULSE, pulse)
        self.model.set_value(iter_, self.COLUMN_ACTIVE, True)

        return True

    def create_model(self):
        self.model = Gtk.ListStore(bool,
                                   GObject.TYPE_INT,
                                   str,
                                   str,
                                   GObject.TYPE_INT,
                                   str,
                                   bool,
                                   bool)

        col = 0
        for bug in data:
            if col == 1 or col == 3:
                icon_name = 'battery-critical-charging-symbolic'
            else:
                icon_name = ''
            if col == 3:
                is_sensitive = False
            else:
                is_sensitive = True

            self.model.append([bug.is_fixed,
                               bug.number,
                               bug.severity,
                               bug.description,
                               0,
                               icon_name,
                               False,
                               is_sensitive])
            col += 1

    def add_columns(self, treeview):
        model = treeview.get_model()

        # column for is_fixed toggle
        renderer = Gtk.CellRendererToggle()
        renderer.connect('toggled', self.is_fixed_toggled, model)

        column = Gtk.TreeViewColumn("Fixed?", renderer,
                                    active=self.COLUMN_FIXED)
        column.set_fixed_width(50)
        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        treeview.append_column(column)

        # column for severities
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Severity", renderer,
                                    text=self.COLUMN_SEVERITY)
        column.set_sort_column_id(self.COLUMN_SEVERITY)
        treeview.append_column(column)

        # column for description
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Description", renderer,
                                    text=self.COLUMN_DESCRIPTION)
        column.set_sort_column_id(self.COLUMN_DESCRIPTION)
        treeview.append_column(column)

        # column for spinner
        renderer = Gtk.CellRendererSpinner()
        column = Gtk.TreeViewColumn("Spinning", renderer,
                                    pulse=self.COLUMN_PULSE,
                                    active=self.COLUMN_ACTIVE)
        column.set_sort_column_id(self.COLUMN_PULSE)
        treeview.append_column(column)

        # column for symbolic icon
        renderer = Gtk.CellRendererPixbuf()
        renderer.props.follow_state = True
        column = Gtk.TreeViewColumn("Symbolic icon", renderer,
                                    icon_name=self.COLUMN_ICON,
                                    sensitive=self.COLUMN_SENSITIVE)
        column.set_sort_column_id(self.COLUMN_ICON)
        treeview.append_column(column)

    def is_fixed_toggled(self, cell, path_str, model):
        # get toggled iter
        iter_ = model.get_iter(path_str)
        is_fixed = model.get_value(iter_, self.COLUMN_FIXED)

        # do something with value
        is_fixed ^= 1

        model.set_value(iter_, self.COLUMN_FIXED, is_fixed)


def main(demoapp=None):
    ListStoreApp()
    Gtk.main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
#
# Copyright (C) 2010 Red Hat, Inc., John (J5) Palmieri <johnp@redhat.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

title = "Color Selector"
description = """
 GtkColorSelection lets the user choose a color. GtkColorSelectionDialog is
 a prebuilt dialog containing a GtkColorSelection.
 """


from gi.repository import Gtk, Gdk


class ColorSelectorApp:
    def __init__(self):
        # FIXME: we should allow Gdk.Color to be allocated without parameters
        #        Also color doesn't seem to work
        self.color = Gdk.RGBA()
        self.color.red = 0
        self.color.blue = 1
        self.color.green = 0
        self.color.alpha = 1

        self.window = Gtk.Window()
        self.window.set_title('Color Selection')
        self.window.set_border_width(8)
        self.window.connect('destroy', lambda w: Gtk.main_quit())

        vbox = Gtk.VBox(homogeneous=False,
                        spacing=8)
        vbox.set_border_width(8)
        self.window.add(vbox)

        # create color swatch area
        frame = Gtk.Frame()
        frame.set_shadow_type(Gtk.ShadowType.IN)
        vbox.pack_start(frame, True, True, 0)

        self.da = Gtk.DrawingArea()
        self.da.connect('draw', self.draw_cb)

        # set a minimum size
        self.da.set_size_request(200, 200)
        # set the color
        self.da.override_background_color(0, self.color)
        frame.add(self.da)

        alignment = Gtk.Alignment(xalign=1.0,
                                  yalign=0.5,
                                  xscale=0.0,
                                  yscale=0.0)

        button = Gtk.Button(label='_Change the above color',
                            use_underline=True)
        alignment.add(button)
        vbox.pack_start(alignment, False, False, 0)

        button.connect('clicked', self.change_color_cb)

        self.window.show_all()

    def draw_cb(self, widget, cairo_ctx):
        style = widget.get_style_context()
        bg_color = style.get_background_color(0)
        Gdk.cairo_set_source_rgba(cairo_ctx, bg_color)
        cairo_ctx.paint()

        return True

    def change_color_cb(self, button):
        dialog = Gtk.ColorSelectionDialog(title='Changing color')
        dialog.set_transient_for(self.window)

        colorsel = dialog.get_color_selection()
        colorsel.set_previous_rgba(self.color)
        colorsel.set_current_rgba(self.color)
        colorsel.set_has_palette(True)

        response = dialog.run()

        if response == Gtk.ResponseType.OK:
            self.color = colorsel.get_current_rgba()
            self.da.override_background_color(0, self.color)

        dialog.destroy()


def main(demoapp=None):
    ColorSelectorApp()
    Gtk.main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
#
# Copyright (C) 2010 Red Hat, Inc., John (J5) Palmieri <johnp@redhat.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

title = "Pickers"
description = """These widgets are mainly intended for use in preference
dialogs. They allow to select colors, fonts, files and directories.
"""

from gi.repository import Gtk


class PickersApp:
    def __init__(self):
        self.window = Gtk.Window(title='Pickers')
        self.window.connect('destroy', Gtk.main_quit)
        self.window.set_border_width(10)

        table = Gtk.Table(n_rows=4, n_columns=2, homogeneous=False)
        table.set_col_spacing(0, 10)
        table.set_row_spacings(3)
        self.window.add(table)
        table.set_border_width(10)

        label = Gtk.Label(label='Color:')
        label.set_alignment(0.0, 0.5)
        picker = Gtk.ColorButton()
        table.attach_defaults(label, 0, 1, 0, 1)
        table.attach_defaults(picker, 1, 2, 0, 1)

        label = Gtk.Label(label='Font:')
        label.set_alignment(0.0, 0.5)
        picker = Gtk.FontButton()
        table.attach_defaults(label, 0, 1, 1, 2)
        table.attach_defaults(picker, 1, 2, 1, 2)

        label = Gtk.Label(label='File:')
        label.set_alignment(0.0, 0.5)
        picker = Gtk.FileChooserButton.new('Pick a File',
                                           Gtk.FileChooserAction.OPEN)
        table.attach_defaults(label, 0, 1, 2, 3)
        table.attach_defaults(picker, 1, 2, 2, 3)

        label = Gtk.Label(label='Folder:')
        label.set_alignment(0.0, 0.5)
        picker = Gtk.FileChooserButton.new('Pick a Folder',
                                           Gtk.FileChooserAction.SELECT_FOLDER)
        table.attach_defaults(label, 0, 1, 3, 4)
        table.attach_defaults(picker, 1, 2, 3, 4)

        self.window.show_all()


def main(demoapp=None):
    PickersApp()
    Gtk.main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
#
# Copyright (C) 2010 Red Hat, Inc., John (J5) Palmieri <johnp@redhat.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

title = "Links"
description = """
GtkLabel can show hyperlinks. The default action is to call gtk_show_uri() on
their URI, but it is possible to override this with a custom handler.
"""

from gi.repository import Gtk


class LinksApp:
    def __init__(self):
        self.window = Gtk.Window()
        self.window.set_title('Links')
        self.window.set_border_width(12)
        self.window.connect('destroy', Gtk.main_quit)

        label = Gtk.Label(label="""Some <a href="http://en.wikipedia.org/wiki/Text"
title="plain text">text</a> may be marked up
as hyperlinks, which can be clicked
or activated via <a href="keynav">keynav</a>""")

        label.set_use_markup(True)
        label.connect("activate-link", self.activate_link)
        self.window.add(label)
        label.show()

        self.window.show()

    def activate_link(self, label, uri):
        if uri == 'keynav':
            parent = label.get_toplevel()
            markup = """The term <i>keynav</i> is a shorthand for
keyboard navigation and refers to the process of using
a program (exclusively) via keyboard input."""
            dialog = Gtk.MessageDialog(transient_for=parent,
                                       destroy_with_parent=True,
                                       message_type=Gtk.MessageType.INFO,
                                       buttons=Gtk.ButtonsType.OK,
                                       text=markup,
                                       use_markup=True)
            dialog.present()
            dialog.connect('response', self.response_cb)

            return True

    def response_cb(self, dialog, response_id):
        dialog.destroy()


def main(demoapp=None):
    LinksApp()
    Gtk.main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

title = "File Tree (GenericTreeModel)"
description = """
This is a file list demo which makes use of the GenericTreeModel python
implementation of the Gtk.TreeModel interface. This demo shows what methods
need to be overridden to provide a valid TreeModel to a TreeView.
"""

import os
import stat
import time
from collections import OrderedDict

import pygtkcompat
pygtkcompat.enable_gtk('3.0')

import gtk


folderxpm = [
    "17 16 7 1",
    "  c #000000",
    ". c #808000",
    "X c yellow",
    "o c #808080",
    "O c #c0c0c0",
    "+ c white",
    "@ c None",
    "@@@@@@@@@@@@@@@@@",
    "@@@@@@@@@@@@@@@@@",
    "@@+XXXX.@@@@@@@@@",
    "@+OOOOOO.@@@@@@@@",
    "@+OXOXOXOXOXOXO. ",
    "@+XOXOXOXOXOXOX. ",
    "@+OXOXOXOXOXOXO. ",
    "@+XOXOXOXOXOXOX. ",
    "@+OXOXOXOXOXOXO. ",
    "@+XOXOXOXOXOXOX. ",
    "@+OXOXOXOXOXOXO. ",
    "@+XOXOXOXOXOXOX. ",
    "@+OOOOOOOOOOOOO. ",
    "@                ",
    "@@@@@@@@@@@@@@@@@",
    "@@@@@@@@@@@@@@@@@"
    ]
folderpb = gtk.gdk.pixbuf_new_from_xpm_data(folderxpm)

filexpm = [
    "12 12 3 1",
    "  c #000000",
    ". c #ffff04",
    "X c #b2c0dc",
    "X        XXX",
    "X ...... XXX",
    "X ......   X",
    "X .    ... X",
    "X ........ X",
    "X .   .... X",
    "X ........ X",
    "X .     .. X",
    "X ........ X",
    "X .     .. X",
    "X ........ X",
    "X          X"
    ]
filepb = gtk.gdk.pixbuf_new_from_xpm_data(filexpm)


class FileTreeModel(gtk.GenericTreeModel):
    __gtype_name__ = 'DemoFileTreeModel'

    column_types = (gtk.gdk.Pixbuf, str, int, str, str)
    column_names = ['Name', 'Size', 'Mode', 'Last Changed']

    def __init__(self, dname=None):
        gtk.GenericTreeModel.__init__(self)
        if not dname:
            self.dirname = os.path.expanduser('~')
        else:
            self.dirname = os.path.abspath(dname)
        self.files = self.build_file_dict(self.dirname)
        return

    def build_file_dict(self, dirname):
        """
        :Returns:
            A dictionary containing the files in the given dirname keyed by filename.
            If the child filename is a sub-directory, the dict value is a dict.
            Otherwise it will be None.
        """
        d = OrderedDict()
        for fname in os.listdir(dirname):
            try:
                filestat = os.stat(os.path.join(dirname, fname))
            except OSError:
                d[fname] = None
            else:
                d[fname] = OrderedDict() if stat.S_ISDIR(filestat.st_mode) else None

        return d

    def get_node_from_treepath(self, path):
        """
        :Returns:
            The node stored at the given tree path in local storage.
        """
        # TreePaths are a series of integer indices so just iterate through them
        # and index values by each integer since we are using an OrderedDict
        if path is None:
            path = []
        node = self.files
        for index in path:
            node = list(node.values())[index]
        return node

    def get_node_from_filepath(self, filepath):
        """
        :Returns:
            The node stored at the given file path in local storage.
        """
        if not filepath:
            return self.files
        node = self.files
        for key in filepath.split(os.path.sep):
            node = node[key]
        return node

    def get_column_names(self):
        return self.column_names[:]

    #
    # GenericTreeModel Implementation
    #

    def on_get_flags(self):
        return 0

    def on_get_n_columns(self):
        return len(self.column_types)

    def on_get_column_type(self, n):
        return self.column_types[n]

    def on_get_path(self, relpath):
        path = []
        node = self.files
        for key in relpath.split(os.path.sep):
            path.append(list(node.keys()).index(key))
            node = node[key]
        return path

    def on_get_value(self, relpath, column):
        fname = os.path.join(self.dirname, relpath)
        try:
            filestat = os.stat(fname)
        except OSError:
            return None
        mode = filestat.st_mode
        if column == 0:
            if stat.S_ISDIR(mode):
                return folderpb
            else:
                return filepb
        elif column == 1:
            return os.path.basename(relpath)
        elif column == 2:
            return filestat.st_size
        elif column == 3:
            return oct(stat.S_IMODE(mode))
        return time.ctime(filestat.st_mtime)

    def on_get_iter(self, path):
        filepath = ''
        value = self.files
        for index in path:
            filepath = os.path.join(filepath, list(value.keys())[index])
            value = list(value.values())[index]
        return filepath

    def on_iter_next(self, filepath):
        parent_path, child_path = os.path.split(filepath)
        parent = self.get_node_from_filepath(parent_path)

        # Index of filepath within its parents child list
        sibling_names = list(parent.keys())
        index = sibling_names.index(child_path)
        try:
            return os.path.join(parent_path, sibling_names[index + 1])
        except IndexError:
            return None

    def on_iter_children(self, filepath):
        if filepath:
            children = list(self.get_node_from_filepath(filepath).keys())
            if children:
                return os.path.join(filepath, children[0])
        elif self.files:
            return list(self.files.keys())[0]

        return None

    def on_iter_has_child(self, filepath):
        return bool(self.get_node_from_filepath(filepath))

    def on_iter_n_children(self, filepath):
        return len(self.get_node_from_filepath(filepath))

    def on_iter_nth_child(self, filepath, n):
        try:
            child = list(self.get_node_from_filepath(filepath).keys())[n]
            if filepath:
                return os.path.join(filepath, child)
            else:
                return child
        except IndexError:
            return None

    def on_iter_parent(self, filepath):
        return os.path.dirname(filepath)

    def on_ref_node(self, filepath):
        value = self.get_node_from_filepath(filepath)
        if value is not None:
            value.update(self.build_file_dict(os.path.join(self.dirname, filepath)))

    def on_unref_node(self, filepath):
        pass


class GenericTreeModelExample:
    def delete_event(self, widget, event, data=None):
        gtk.main_quit()
        return False

    def __init__(self):
        # Create a new window
        self.window = gtk.Window(type=gtk.WINDOW_TOPLEVEL)
        self.window.set_size_request(300, 200)
        self.window.connect("delete_event", self.delete_event)

        self.listmodel = FileTreeModel()

        # create the TreeView
        self.treeview = gtk.TreeView()

        # create the TreeViewColumns to display the data
        column_names = self.listmodel.get_column_names()
        self.tvcolumn = [None] * len(column_names)
        cellpb = gtk.CellRendererPixbuf()
        self.tvcolumn[0] = gtk.TreeViewColumn(column_names[0],
                                              cellpb, pixbuf=0)
        cell = gtk.CellRendererText()
        self.tvcolumn[0].pack_start(cell, False)
        self.tvcolumn[0].add_attribute(cell, 'text', 1)
        self.treeview.append_column(self.tvcolumn[0])
        for n in range(1, len(column_names)):
            cell = gtk.CellRendererText()
            if n == 1:
                cell.set_property('xalign', 1.0)
            self.tvcolumn[n] = gtk.TreeViewColumn(column_names[n],
                                                  cell, text=n + 1)
            self.treeview.append_column(self.tvcolumn[n])

        self.scrolledwindow = gtk.ScrolledWindow()
        self.scrolledwindow.add(self.treeview)
        self.window.add(self.scrolledwindow)
        self.treeview.set_model(self.listmodel)
        self.window.set_title(self.listmodel.dirname)
        self.window.show_all()


def main(demoapp=None):
    demo = GenericTreeModelExample()
    demo
    gtk.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
#
# Copyright (C) 2013 Gian Mario Tagliaretti <gianmt@gnome.org>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

title = "CSS Theming/Multiple Backgrounds"
description = """
Gtk themes are written using CSS. Every widget is build of multiple items
that you can style very similarly to a regular website.
"""

from gi.repository import Gtk, Gdk, Pango, Gio, GLib


class CSSMultiplebgsApp:
    def __init__(self, demoapp):
        self.demoapp = demoapp
        #: Store the last successful parsing of the css so we can revert
        #: this in case of an error.
        self.last_good_text = ''
        #: Set when we receive a parsing-error callback. This is needed
        #: to handle logic after a parsing-error callback which does not raise
        #: an exception with provider.load_from_data()
        self.last_error_code = 0

        self.window = Gtk.Window()
        self.window.set_title('CSS Multiplebgs')
        self.window.set_default_size(400, 300)
        self.window.set_border_width(10)
        self.window.connect('destroy', lambda w: Gtk.main_quit())

        overlay = Gtk.Overlay()
        overlay.add_events(Gdk.EventMask.ENTER_NOTIFY_MASK |
                           Gdk.EventMask.LEAVE_NOTIFY_MASK |
                           Gdk.EventMask.POINTER_MOTION_MASK)

        self.infobar = Gtk.InfoBar()
        self.infolabel = Gtk.Label()
        self.infobar.get_content_area().pack_start(self.infolabel, False, False, 0)
        self.infobar.set_message_type(Gtk.MessageType.WARNING)

        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        box.pack_start(overlay, expand=True, fill=True, padding=0)
        box.pack_start(self.infobar, expand=False, fill=True, padding=0)
        self.window.add(box)

        canvas = Gtk.DrawingArea()
        canvas.set_name("canvas")
        canvas.connect("draw", self.drawing_area_draw)
        overlay.add(canvas)

        button = Gtk.Button()
        button.add_events(Gdk.EventMask.ENTER_NOTIFY_MASK |
                          Gdk.EventMask.LEAVE_NOTIFY_MASK |
                          Gdk.EventMask.POINTER_MOTION_MASK)
        button.set_name("bricks-button")
        button.set_halign(Gtk.Align.CENTER)
        button.set_valign(Gtk.Align.CENTER)
        button.set_size_request(250, 84)
        overlay.add_overlay(button)

        paned = Gtk.Paned(orientation=Gtk.Orientation.VERTICAL)
        overlay.add_overlay(paned)

        # We need a filler so we get a handle
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        paned.add(box)

        buffer = Gtk.TextBuffer()
        buffer.create_tag(tag_name="warning", underline=Pango.Underline.SINGLE)
        buffer.create_tag(tag_name="error", underline=Pango.Underline.ERROR)

        provider = Gtk.CssProvider()

        buffer.connect("changed", self.css_text_changed, provider)
        provider.connect("parsing-error", self.show_parsing_error, buffer)

        textview = Gtk.TextView()
        textview.set_buffer(buffer)

        scrolled = Gtk.ScrolledWindow()
        scrolled.add(textview)
        paned.add(scrolled)

        bytes = Gio.resources_lookup_data("/css_multiplebgs/css_multiplebgs.css", 0)
        buffer.set_text(bytes.get_data().decode('utf-8'))

        self.apply_css(self.window, provider)
        self.window.show_all()
        self.infobar.hide()

    def drawing_area_draw(self, widget, cairo_t):
        context = widget.get_style_context()
        Gtk.render_background(context, cairo_t, 0, 0,
                              widget.get_allocated_width(),
                              widget.get_allocated_height())

        Gtk.render_frame(context, cairo_t, 0, 0,
                         widget.get_allocated_width(),
                         widget.get_allocated_height())

    def apply_css(self, widget, provider):
        Gtk.StyleContext.add_provider(widget.get_style_context(),
                                      provider,
                                      Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

        if isinstance(widget, Gtk.Container):
            widget.forall(self.apply_css, provider)

    def show_parsing_error(self, provider, section, error, buffer):
        start = buffer.get_iter_at_line_index(section.get_start_line(),
                                              section.get_start_position())

        end = buffer.get_iter_at_line_index(section.get_end_line(),
                                            section.get_end_position())

        if error.code == Gtk.CssProviderError.DEPRECATED:
            tag_name = "warning"
        else:
            tag_name = "error"
        self.last_error_code = error.code

        self.infolabel.set_text(error.message)
        self.infobar.show_all()

        buffer.apply_tag_by_name(tag_name, start, end)

    def css_text_changed(self, buffer, provider):
        start = buffer.get_start_iter()
        end = buffer.get_end_iter()
        buffer.remove_all_tags(start, end)

        text = buffer.get_text(start, end, False).encode('utf-8')

        # Ignore CSS errors as they are shown by highlighting
        try:
            provider.load_from_data(text)
        except GLib.GError as e:
            if e.domain != 'gtk-css-provider-error-quark':
                raise e

        # If the parsing-error callback is ever run (even in the case of warnings)
        # load the last good css text that ran without any warnings. Otherwise
        # we may have a discrepancy in "last_good_text" vs the current buffer
        # causing section.get_start_position() to give back an invalid position
        # for the editor buffer.
        if self.last_error_code:
            provider.load_from_data(self.last_good_text)
            self.last_error_code = 0
        else:
            self.last_good_text = text
            self.infobar.hide()

        Gtk.StyleContext.reset_widgets(Gdk.Screen.get_default())


def main(demoapp=None):
    CSSMultiplebgsApp(demoapp)
    Gtk.main()


if __name__ == '__main__':
    import os
    base_path = os.path.abspath(os.path.dirname(__file__))
    resource_path = os.path.join(base_path, '../data/demo.gresource')
    resource = Gio.Resource.load(resource_path)

    # FIXME: method register() should be without the underscore
    # FIXME: see https://bugzilla.gnome.org/show_bug.cgi?id=684319
    resource._register()
    main()



# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

from __future__ import absolute_import

import os
import gc
import unittest
import tempfile
import socket

import pytest

import gi
from gi.repository import GLib
from gi._compat import long_, integer_types

from .helper import capture_gi_deprecation_warnings


def test_io_add_watch_get_args():
    get_args = GLib._io_add_watch_get_args
    func = lambda: None

    # create a closed channel for testing
    fd, fn = tempfile.mkstemp()
    os.close(fd)
    try:
        chan = GLib.IOChannel(filename=fn)
        chan.shutdown(True)
    finally:
        os.remove(fn)

    # old way
    with capture_gi_deprecation_warnings():
        assert get_args(chan, GLib.IOCondition.IN, func) == (
            chan, 0, GLib.IOCondition.IN, func, tuple())

        with pytest.raises(TypeError):
            get_args(chan, GLib.IOCondition.IN, object())

    # new way
    prio = GLib.PRIORITY_DEFAULT
    with capture_gi_deprecation_warnings():
        assert get_args(chan, prio, GLib.IOCondition.IN, func, 99) == \
            (chan, prio, GLib.IOCondition.IN, func, (99,))

        with pytest.raises(TypeError):
            assert get_args(chan, prio, GLib.IOCondition.IN)

        with pytest.raises(TypeError):
            assert get_args(chan, prio, 99)


@pytest.mark.skipif(os.name != "nt", reason="windows only")
def test_io_add_watch_get_args_win32_socket():
    get_args = GLib._io_add_watch_get_args

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    func = lambda: None
    prio = GLib.PRIORITY_DEFAULT
    chan = get_args(s, prio, GLib.IOCondition.IN, func)[0]
    assert isinstance(chan, GLib.IOChannel)
    chan.shutdown(False)


def test_threads_init():
    with capture_gi_deprecation_warnings() as w:
        GLib.threads_init()
    assert len(w)


def test_gerror_matches():
    e = GLib.Error(domain=42, code=24)
    assert e.matches(42, 24)


def test_timeout_add_seconds():
    h = GLib.timeout_add_seconds(
        100, lambda *x: None, 1, 2, 3, priority=GLib.PRIORITY_HIGH_IDLE)
    GLib.source_remove(h)


def test_iochannel():
    with pytest.raises(TypeError):
        GLib.IOChannel()


def test_iochannel_write():
    fd, fn = tempfile.mkstemp()
    os.close(fd)
    chan = GLib.IOChannel(filename=fn, mode="r+")
    try:
        assert chan.write(b"foo", 2) == 2
        chan.seek(0)
        assert chan.read() == b"fo"
    finally:
        chan.shutdown(True)


@pytest.mark.skipif(os.name == "nt", reason="unix only")
def test_has_unix_signal_add():
    with capture_gi_deprecation_warnings():
        GLib.unix_signal_add == GLib.unix_signal_add_full


@pytest.mark.skipif(os.name != "nt", reason="windows only")
def test_iochannel_win32():
    fd, fn = tempfile.mkstemp()
    closed = False
    try:
        channel = GLib.IOChannel(hwnd=fd)
        try:
            assert channel.read() == b""
        finally:
            closed = True
            channel.shutdown(True)
    finally:
        if not closed:
            os.close(fd)
        os.remove(fn)


class TestGVariant(unittest.TestCase):
    def test_create_simple(self):
        variant = GLib.Variant('i', 42)
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertEqual(variant.get_int32(), 42)

        variant = GLib.Variant('s', '')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertEqual(variant.get_string(), '')

        variant = GLib.Variant('s', 'hello')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertEqual(variant.get_string(), 'hello')

    def test_simple_invalid_ops(self):
        variant = GLib.Variant('i', 42)
        with pytest.raises(TypeError):
            len(variant)

        with pytest.raises(TypeError):
            variant[0]

        with pytest.raises(TypeError):
            variant.keys()

    def test_create_variant(self):
        variant = GLib.Variant('v', GLib.Variant('i', 42))
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertTrue(isinstance(variant.get_variant(), GLib.Variant))
        self.assertEqual(variant.get_type_string(), 'v')
        self.assertEqual(variant.get_variant().get_type_string(), 'i')
        self.assertEqual(variant.get_variant().get_int32(), 42)

        variant = GLib.Variant('v', GLib.Variant('v', GLib.Variant('i', 42)))
        self.assertEqual(variant.get_type_string(), 'v')
        self.assertEqual(variant.get_variant().get_type_string(), 'v')
        self.assertEqual(variant.get_variant().get_variant().get_type_string(), 'i')
        self.assertEqual(variant.get_variant().get_variant().get_int32(), 42)

    def test_create_tuple(self):
        variant = GLib.Variant('()', ())
        self.assertEqual(variant.get_type_string(), '()')
        self.assertEqual(variant.n_children(), 0)

        variant = GLib.Variant('(i)', (3,))
        self.assertEqual(variant.get_type_string(), '(i)')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertEqual(variant.n_children(), 1)
        self.assertTrue(isinstance(variant.get_child_value(0), GLib.Variant))
        self.assertEqual(variant.get_child_value(0).get_int32(), 3)

        variant = GLib.Variant('(ss)', ('mec', 'mac'))
        self.assertEqual(variant.get_type_string(), '(ss)')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(0), GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(1), GLib.Variant))
        self.assertEqual(variant.get_child_value(0).get_string(), 'mec')
        self.assertEqual(variant.get_child_value(1).get_string(), 'mac')

        # nested tuples
        variant = GLib.Variant('((si)(ub))', (('hello', -1), (42, True)))
        self.assertEqual(variant.get_type_string(), '((si)(ub))')
        self.assertEqual(variant.unpack(), (('hello', -1), (long_(42), True)))

    def test_new_tuple_sink(self):
        # https://bugzilla.gnome.org/show_bug.cgi?id=735166
        variant = GLib.Variant.new_tuple(GLib.Variant.new_tuple())
        del variant
        gc.collect()

    def test_create_dictionary(self):
        variant = GLib.Variant('a{si}', {})
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertEqual(variant.get_type_string(), 'a{si}')
        self.assertEqual(variant.n_children(), 0)

        variant = GLib.Variant('a{si}', {'': 1, 'key1': 2, 'key2': 3})
        self.assertEqual(variant.get_type_string(), 'a{si}')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(0), GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(1), GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(2), GLib.Variant))
        self.assertEqual(variant.unpack(), {'': 1, 'key1': 2, 'key2': 3})

        # nested dictionaries
        variant = GLib.Variant('a{sa{si}}', {})
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertEqual(variant.get_type_string(), 'a{sa{si}}')
        self.assertEqual(variant.n_children(), 0)

        d = {'': {'': 1, 'keyn1': 2},
             'key1': {'key11': 11, 'key12': 12}}
        variant = GLib.Variant('a{sa{si}}', d)
        self.assertEqual(variant.get_type_string(), 'a{sa{si}}')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertEqual(variant.unpack(), d)

        # init with an iterable
        variant = GLib.Variant('a{si}', [("foo", 2)])
        assert variant.unpack() == {'foo': 2}

        with pytest.raises(TypeError):
            GLib.Variant('a{si}', [("foo",)])
        with pytest.raises(TypeError):
            GLib.Variant('a{si}', [("foo", 1, 2)])

    def test_create_array(self):
        variant = GLib.Variant('ai', [])
        self.assertEqual(variant.get_type_string(), 'ai')
        self.assertEqual(variant.n_children(), 0)

        variant = GLib.Variant('ai', [1, 2])
        self.assertEqual(variant.get_type_string(), 'ai')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(0), GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(1), GLib.Variant))
        self.assertEqual(variant.get_child_value(0).get_int32(), 1)
        self.assertEqual(variant.get_child_value(1).get_int32(), 2)

        variant = GLib.Variant('as', [])
        self.assertEqual(variant.get_type_string(), 'as')
        self.assertEqual(variant.n_children(), 0)

        variant = GLib.Variant('as', [''])
        self.assertEqual(variant.get_type_string(), 'as')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(0), GLib.Variant))
        self.assertEqual(variant.get_child_value(0).get_string(), '')

        variant = GLib.Variant('as', ['hello', 'world'])
        self.assertEqual(variant.get_type_string(), 'as')
        self.assertTrue(isinstance(variant, GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(0), GLib.Variant))
        self.assertTrue(isinstance(variant.get_child_value(1), GLib.Variant))
        self.assertEqual(variant.get_child_value(0).get_string(), 'hello')
        self.assertEqual(variant.get_child_value(1).get_string(), 'world')

        # nested arrays
        variant = GLib.Variant('aai', [])
        self.assertEqual(variant.get_type_string(), 'aai')
        self.assertEqual(variant.n_children(), 0)

        variant = GLib.Variant('aai', [[]])
        self.assertEqual(variant.get_type_string(), 'aai')
        self.assertEqual(variant.n_children(), 1)
        self.assertEqual(variant.get_child_value(0).n_children(), 0)

        variant = GLib.Variant('aai', [[1, 2], [3, 4, 5]])
        self.assertEqual(variant.get_type_string(), 'aai')
        self.assertEqual(variant.unpack(), [[1, 2], [3, 4, 5]])

    def test_create_array_guchar(self):
        variant = GLib.Variant('ay', [97, 97, 97])
        assert variant.unpack() == [97, 97, 97]

        variant = GLib.Variant('ay', b'aaa')
        assert variant.unpack() == [97, 97, 97]

        variant = GLib.Variant('ay', iter([1, 2, 3]))
        assert variant.unpack() == [1, 2, 3]

        with self.assertRaises(TypeError):
            GLib.Variant('ay', u'aaa')

        with self.assertRaises(TypeError):
            GLib.Variant('ay', object())

    def test_create_maybe(self):
        variant = GLib.Variant('mai', None)
        self.assertEqual(variant.get_type_string(), 'mai')
        self.assertEqual(variant.n_children(), 0)
        self.assertEqual(variant.unpack(), None)

        variant = GLib.Variant('mai', [])
        self.assertEqual(variant.get_type_string(), 'mai')
        self.assertEqual(variant.n_children(), 1)

        variant = GLib.Variant('mami', [None])
        self.assertEqual(variant.get_type_string(), 'mami')
        self.assertEqual(variant.n_children(), 1)

        variant = GLib.Variant('mami', [None, 13, None])
        self.assertEqual(variant.get_type_string(), 'mami')
        self.assertEqual(variant.n_children(), 1)
        array = variant.get_child_value(0)
        self.assertEqual(array.n_children(), 3)

        element = array.get_child_value(0)
        self.assertEqual(element.n_children(), 0)
        element = array.get_child_value(1)
        self.assertEqual(element.n_children(), 1)
        self.assertEqual(element.get_child_value(0).get_int32(), 13)
        element = array.get_child_value(2)
        self.assertEqual(element.n_children(), 0)

    def test_create_complex(self):
        variant = GLib.Variant('(as)', ([],))
        self.assertEqual(variant.get_type_string(), '(as)')
        self.assertEqual(variant.n_children(), 1)
        self.assertEqual(variant.get_child_value(0).n_children(), 0)

        variant = GLib.Variant('(as)', ([''],))
        self.assertEqual(variant.get_type_string(), '(as)')
        self.assertEqual(variant.n_children(), 1)
        self.assertEqual(variant.get_child_value(0).n_children(), 1)
        self.assertEqual(variant.get_child_value(0).get_child_value(0).get_string(), '')

        variant = GLib.Variant('(as)', (['hello'],))
        self.assertEqual(variant.get_type_string(), '(as)')
        self.assertEqual(variant.n_children(), 1)
        self.assertEqual(variant.get_child_value(0).n_children(), 1)
        self.assertEqual(variant.get_child_value(0).get_child_value(0).get_string(), 'hello')

        variant = GLib.Variant('a(ii)', [])
        self.assertEqual(variant.get_type_string(), 'a(ii)')
        self.assertEqual(variant.n_children(), 0)

        variant = GLib.Variant('a(ii)', [(5, 6)])
        self.assertEqual(variant.get_type_string(), 'a(ii)')
        self.assertEqual(variant.n_children(), 1)
        self.assertEqual(variant.get_child_value(0).n_children(), 2)
        self.assertEqual(variant.get_child_value(0).get_child_value(0).get_int32(), 5)
        self.assertEqual(variant.get_child_value(0).get_child_value(1).get_int32(), 6)

        variant = GLib.Variant('(a(ii))', ([],))
        self.assertEqual(variant.get_type_string(), '(a(ii))')
        self.assertEqual(variant.n_children(), 1)
        self.assertEqual(variant.get_child_value(0).n_children(), 0)

        variant = GLib.Variant('(a(ii))', ([(5, 6)],))
        self.assertEqual(variant.get_type_string(), '(a(ii))')
        self.assertEqual(variant.n_children(), 1)
        self.assertEqual(variant.get_child_value(0).n_children(), 1)
        self.assertEqual(variant.get_child_value(0).get_child_value(0).n_children(), 2)
        self.assertEqual(variant.get_child_value(0).get_child_value(0).get_child_value(0).get_int32(), 5)
        self.assertEqual(variant.get_child_value(0).get_child_value(0).get_child_value(1).get_int32(), 6)

        obj = {'a1': (1, True), 'a2': (2, False)}
        variant = GLib.Variant('a{s(ib)}', obj)
        self.assertEqual(variant.get_type_string(), 'a{s(ib)}')
        self.assertEqual(variant.unpack(), obj)

        obj = {'a1': (1, GLib.Variant('b', True)), 'a2': (2, GLib.Variant('y', 255))}
        variant = GLib.Variant('a{s(iv)}', obj)
        self.assertEqual(variant.get_type_string(), 'a{s(iv)}')
        self.assertEqual(variant.unpack(), {'a1': (1, True), 'a2': (2, 255)})

        obj = (1, {'a': {'a1': True, 'a2': False},
                   'b': {'b1': False},
                   'c': {}
                  },
               'foo')
        variant = GLib.Variant('(ia{sa{sb}}s)', obj)
        self.assertEqual(variant.get_type_string(), '(ia{sa{sb}}s)')
        self.assertEqual(variant.unpack(), obj)

        obj = {"frequency": GLib.Variant('t', 738000000),
               "hierarchy": GLib.Variant('i', 0),
               "bandwidth": GLib.Variant('x', 8),
               "code-rate-hp": GLib.Variant('d', 2.0 / 3.0),
               "constellation": GLib.Variant('s', "QAM16"),
               "guard-interval": GLib.Variant('u', 4)}
        variant = GLib.Variant('a{sv}', obj)
        self.assertEqual(variant.get_type_string(), 'a{sv}')
        self.assertEqual(variant.unpack(),
                         {"frequency": 738000000,
                          "hierarchy": 0,
                          "bandwidth": 8,
                          "code-rate-hp": 2.0 / 3.0,
                          "constellation": "QAM16",
                          "guard-interval": 4
                         })

    def test_create_errors(self):
        # excess arguments
        self.assertRaises(TypeError, GLib.Variant, 'i', 42, 3)
        self.assertRaises(TypeError, GLib.Variant, '(i)', (42, 3))

        # not enough arguments
        self.assertRaises(TypeError, GLib.Variant, '(ii)', (42,))

        # data type mismatch
        self.assertRaises(TypeError, GLib.Variant, 'i', 'hello')
        self.assertRaises(TypeError, GLib.Variant, 's', 42)
        self.assertRaises(TypeError, GLib.Variant, '(ss)', 'mec', 'mac')
        self.assertRaises(TypeError, GLib.Variant, '(s)', 'hello')

        # invalid format string
        self.assertRaises(TypeError, GLib.Variant, 'Q', 1)

        # invalid types
        self.assertRaises(TypeError, GLib.Variant, '(ii', (42, 3))
        self.assertRaises(TypeError, GLib.Variant, '(ii))', (42, 3))
        self.assertRaises(TypeError, GLib.Variant, 'a{si', {})
        self.assertRaises(TypeError, GLib.Variant, 'a{si}}', {})
        self.assertRaises(TypeError, GLib.Variant, 'a{iii}', {})

    def test_unpack(self):
        # simple values
        res = GLib.Variant.new_int32(-42).unpack()
        self.assertEqual(res, -42)

        res = GLib.Variant.new_uint64(34359738368).unpack()
        self.assertEqual(res, 34359738368)

        res = GLib.Variant.new_boolean(True).unpack()
        self.assertEqual(res, True)

        res = GLib.Variant.new_object_path('/foo/Bar').unpack()
        self.assertEqual(res, '/foo/Bar')

        # variant
        res = GLib.Variant('v', GLib.Variant.new_int32(-42)).unpack()
        self.assertEqual(res, -42)

        GLib.Variant('v', GLib.Variant('v', GLib.Variant('i', 42)))
        self.assertEqual(res, -42)

        # tuple
        res = GLib.Variant.new_tuple(GLib.Variant.new_int32(-1),
                                     GLib.Variant.new_string('hello')).unpack()
        self.assertEqual(res, (-1, 'hello'))

        # array
        vb = GLib.VariantBuilder.new(gi._gi.variant_type_from_string('ai'))
        vb.add_value(GLib.Variant.new_int32(-1))
        vb.add_value(GLib.Variant.new_int32(3))
        res = vb.end().unpack()
        self.assertEqual(res, [-1, 3])

        # dictionary
        res = GLib.Variant('a{si}', {'key1': 1, 'key2': 2}).unpack()
        self.assertEqual(res, {'key1': 1, 'key2': 2})

        # maybe
        v = GLib.Variant('mi', 1)
        self.assertEqual(v.unpack(), 1)
        v = GLib.Variant('mi', None)
        self.assertEqual(v.unpack(), None)
        v = GLib.Variant('mai', [])
        self.assertEqual(v.unpack(), [])
        v = GLib.Variant('m()', ())
        self.assertEqual(v.unpack(), ())
        v = GLib.Variant('mami', [None, 1, None])
        self.assertEqual(v.unpack(), [None, 1, None])

    def test_iteration(self):
        # array index access
        vb = GLib.VariantBuilder.new(gi._gi.variant_type_from_string('ai'))
        vb.add_value(GLib.Variant.new_int32(-1))
        vb.add_value(GLib.Variant.new_int32(3))
        v = vb.end()

        self.assertEqual(len(v), 2)
        self.assertEqual(v[0], -1)
        self.assertEqual(v[1], 3)
        self.assertEqual(v[-1], 3)
        self.assertEqual(v[-2], -1)
        self.assertRaises(IndexError, v.__getitem__, 2)
        self.assertRaises(IndexError, v.__getitem__, -3)
        self.assertRaises(ValueError, v.__getitem__, 'a')

        # array iteration
        self.assertEqual([x for x in v], [-1, 3])
        self.assertEqual(list(v), [-1, 3])

        # tuple index access
        v = GLib.Variant.new_tuple(GLib.Variant.new_int32(-1),
                                   GLib.Variant.new_string('hello'))
        self.assertEqual(len(v), 2)
        self.assertEqual(v[0], -1)
        self.assertEqual(v[1], 'hello')
        self.assertEqual(v[-1], 'hello')
        self.assertEqual(v[-2], -1)
        self.assertRaises(IndexError, v.__getitem__, 2)
        self.assertRaises(IndexError, v.__getitem__, -3)
        self.assertRaises(ValueError, v.__getitem__, 'a')

        # tuple iteration
        self.assertEqual([x for x in v], [-1, 'hello'])
        self.assertEqual(tuple(v), (-1, 'hello'))

        # dictionary index access
        vsi = GLib.Variant('a{si}', {'key1': 1, 'key2': 2})
        vis = GLib.Variant('a{is}', {1: 'val1', 5: 'val2'})

        self.assertEqual(len(vsi), 2)
        self.assertEqual(vsi['key1'], 1)
        self.assertEqual(vsi['key2'], 2)
        self.assertRaises(KeyError, vsi.__getitem__, 'unknown')

        self.assertEqual(len(vis), 2)
        self.assertEqual(vis[1], 'val1')
        self.assertEqual(vis[5], 'val2')
        self.assertRaises(KeyError, vsi.__getitem__, 3)

        # dictionary iteration
        self.assertEqual(set(vsi.keys()), set(['key1', 'key2']))
        self.assertEqual(set(vis.keys()), set([1, 5]))

        # string index access
        v = GLib.Variant('s', 'hello')
        self.assertEqual(len(v), 5)
        self.assertEqual(v[0], 'h')
        self.assertEqual(v[4], 'o')
        self.assertEqual(v[-1], 'o')
        self.assertEqual(v[-5], 'h')
        self.assertRaises(IndexError, v.__getitem__, 5)
        self.assertRaises(IndexError, v.__getitem__, -6)

        # string iteration
        self.assertEqual([x for x in v], ['h', 'e', 'l', 'l', 'o'])

    def test_split_signature(self):
        self.assertEqual(GLib.Variant.split_signature('()'), [])

        self.assertEqual(GLib.Variant.split_signature('s'), ['s'])

        self.assertEqual(GLib.Variant.split_signature('as'), ['as'])

        self.assertEqual(GLib.Variant.split_signature('(s)'), ['s'])

        self.assertEqual(GLib.Variant.split_signature('(iso)'), ['i', 's', 'o'])

        self.assertEqual(GLib.Variant.split_signature('(s(ss)i(ii))'),
                         ['s', '(ss)', 'i', '(ii)'])

        self.assertEqual(GLib.Variant.split_signature('(as)'), ['as'])

        self.assertEqual(GLib.Variant.split_signature('(s(ss)iaiaasa(ii))'),
                         ['s', '(ss)', 'i', 'ai', 'aas', 'a(ii)'])

        self.assertEqual(GLib.Variant.split_signature('(a{iv}(ii)((ss)a{s(ss)}))'),
                         ['a{iv}', '(ii)', '((ss)a{s(ss)})'])

    def test_hash(self):
        v1 = GLib.Variant('s', 'somestring')
        v2 = GLib.Variant('s', 'somestring')
        v3 = GLib.Variant('s', 'somestring2')

        self.assertTrue(v2 in set([v1, v3]))
        self.assertTrue(v2 in frozenset([v1, v3]))
        self.assertTrue(v2 in {v1: '1', v3: '2'})

    def test_compare(self):
        # Check if identical GVariant are equal

        def assert_equal(vtype, value):
            self.assertEqual(GLib.Variant(vtype, value), GLib.Variant(vtype, value))

        def assert_not_equal(vtype1, value1, vtype2, value2):
            self.assertNotEqual(GLib.Variant(vtype1, value1), GLib.Variant(vtype2, value2))

        numbers = ['y', 'n', 'q', 'i', 'u', 'x', 't', 'h', 'd']
        for num in numbers:
            assert_equal(num, 42)
            assert_not_equal(num, 42, num, 41)
            assert_not_equal(num, 42, 's', '42')

        assert_equal('s', 'something')
        assert_not_equal('s', 'something', 's', 'somethingelse')
        assert_not_equal('s', 'something', 'i', 1234)

        assert_equal('g', 'dustybinqhogx')
        assert_not_equal('g', 'dustybinqhogx', 'g', 'dustybin')
        assert_not_equal('g', 'dustybinqhogx', 'i', 1234)

        assert_equal('o', '/dev/null')
        assert_not_equal('o', '/dev/null', 'o', '/dev/zero')
        assert_not_equal('o', '/dev/null', 'i', 1234)

        assert_equal('(s)', ('strtuple',))
        assert_not_equal('(s)', ('strtuple',), '(s)', ('strtuple2',))

        assert_equal('a{si}', {'str': 42})
        assert_not_equal('a{si}', {'str': 42}, 'a{si}', {'str': 43})

        assert_equal('v', GLib.Variant('i', 42))
        assert_not_equal('v', GLib.Variant('i', 42), 'v', GLib.Variant('i', 43))

        assert GLib.Variant('i', 42) != object()
        assert not GLib.Variant('i', 42) == object()

    def test_bool(self):
        # Check if the GVariant bool matches the unpacked Pythonic bool

        def assert_equals_bool(vtype, value):
            self.assertEqual(bool(GLib.Variant(vtype, value)), bool(value))

        # simple values
        assert_equals_bool('b', True)
        assert_equals_bool('b', False)

        numbers = ['y', 'n', 'q', 'i', 'u', 'x', 't', 'h', 'd']
        for number in numbers:
            assert_equals_bool(number, 0)
            assert_equals_bool(number, 1)

        assert_equals_bool('s', '')
        assert_equals_bool('g', '')
        assert_equals_bool('s', 'something')
        assert_equals_bool('o', '/dev/null')
        assert_equals_bool('g', 'dustybinqhogx')

        # arrays
        assert_equals_bool('ab', [True])
        assert_equals_bool('ab', [False])
        for number in numbers:
            assert_equals_bool('a' + number, [])
            assert_equals_bool('a' + number, [0])
        assert_equals_bool('as', [])
        assert_equals_bool('as', [''])
        assert_equals_bool('ao', [])
        assert_equals_bool('ao', ['/'])
        assert_equals_bool('ag', [])
        assert_equals_bool('ag', [''])
        assert_equals_bool('aai', [[]])

        # tuples
        assert_equals_bool('()', ())
        for number in numbers:
            assert_equals_bool('(' + number + ')', (0,))
        assert_equals_bool('(s)', ('',))
        assert_equals_bool('(o)', ('/',))
        assert_equals_bool('(g)', ('',))
        assert_equals_bool('(())', ((),))

        # dictionaries
        assert_equals_bool('a{si}', {})
        assert_equals_bool('a{si}', {'': 0})

        # complex types, always True
        assert_equals_bool('(as)', ([],))
        assert_equals_bool('a{s(i)}', {'': (0,)})

        # variant types, recursive unpacking
        assert_equals_bool('v', GLib.Variant('i', 0))
        assert_equals_bool('v', GLib.Variant('i', 1))

        # maybe types
        assert_equals_bool('mi', 42)
        assert_equals_bool('mi', 0)
        assert_equals_bool('mi', None)

    def test_repr(self):
        # with C constructor
        v = GLib.Variant.new_uint32(42)
        self.assertEqual(repr(v), "GLib.Variant('u', 42)")

        # with override constructor
        v = GLib.Variant('(is)', (1, 'somestring'))
        self.assertEqual(repr(v), "GLib.Variant('(is)', (1, 'somestring'))")

    def test_str(self):
        # with C constructor
        v = GLib.Variant.new_uint32(42)
        self.assertEqual(str(v), 'uint32 42')

        # with override constructor
        v = GLib.Variant('(is)', (1, 'somestring'))
        self.assertEqual(str(v), "(1, 'somestring')")

    def test_parse_error(self):
        # This test doubles as a test for GLib.Error marshaling.
        source_str = 'abc'
        with self.assertRaises(GLib.Error) as context:
            GLib.Variant.parse(None, source_str, None, None)
        e = context.exception
        text = GLib.Variant.parse_error_print_context(e, source_str)
        self.assertTrue(source_str in text)

    def test_parse_error_exceptions(self):
        source_str = 'abc'
        self.assertRaisesRegexp(TypeError, 'Must be GLib.Error, not int',
                                GLib.Variant.parse_error_print_context,
                                42, source_str)

        gerror = GLib.Error(message=42)  # not a string
        self.assertRaisesRegexp(TypeError, ".*Must be string, not int.*",
                                GLib.Variant.parse_error_print_context,
                                gerror, source_str)

        gerror = GLib.Error(domain=42)  # not a string
        self.assertRaisesRegexp(TypeError, ".*Must be string, not int.*",
                                GLib.Variant.parse_error_print_context,
                                gerror, source_str)

        gerror = GLib.Error(code='not an int')
        self.assertRaisesRegexp(TypeError, ".*Must be number, not str.*",
                                GLib.Variant.parse_error_print_context,
                                gerror, source_str)

        gerror = GLib.Error(code=GLib.MAXUINT)
        self.assertRaisesRegexp(OverflowError,
                                ".*not in range.*",
                                GLib.Variant.parse_error_print_context,
                                gerror, source_str)


class TestConstants(unittest.TestCase):

    def test_basic_types_limits(self):
        self.assertTrue(isinstance(GLib.MINFLOAT, float))
        self.assertTrue(isinstance(GLib.MAXLONG, integer_types))



#!/usr/bin/env python
# -*- Mode: Python; py-indent-offset: 4 -*-
# vim: tabstop=4 shiftwidth=4 expandtab
#
# Copyright (C) 2010 Red Hat, Inc., John (J5) Palmieri <johnp@redhat.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

title = "Combo boxes"
description = """
The ComboBox widget allows to select one option out of a list.
The ComboBoxEntry additionally allows the user to enter a value
that is not in the list of options.

How the options are displayed is controlled by cell renderers.
 """


from gi.repository import Gtk, Gdk, GdkPixbuf, GLib, GObject


(PIXBUF_COL,
 TEXT_COL) = range(2)


class MaskEntry(Gtk.Entry):
    __gtype_name__ = 'MaskEntry'

    def __init__(self, mask=None):
        self.mask = mask
        super(MaskEntry, self).__init__()

        self.connect('changed', self.changed_cb)

        self.error_color = Gdk.RGBA()
        self.error_color.red = 1.0
        self.error_color.green = 0.9
        self.error_color_blue = 0.9
        self.error_color.alpha = 1.0

        # workaround since override_color doesn't accept None yet
        style_ctx = self.get_style_context()
        self.normal_color = style_ctx.get_color(0)

    def set_background(self):
        if self.mask:
            if not GLib.regex_match_simple(self.mask,
                                           self.get_text(), 0, 0):
                self.override_color(0, self.error_color)
                return

        self.override_color(0, self.normal_color)

    def changed_cb(self, entry):
        self.set_background()


class ComboboxApp:
    def __init__(self, demoapp):
        self.demoapp = demoapp

        self.window = Gtk.Window()
        self.window.set_title('Combo boxes')
        self.window.set_border_width(10)
        self.window.connect('destroy', lambda w: Gtk.main_quit())

        vbox = Gtk.VBox(homogeneous=False, spacing=2)
        self.window.add(vbox)

        frame = Gtk.Frame(label='Some stock icons')
        vbox.pack_start(frame, False, False, 0)

        box = Gtk.VBox(homogeneous=False, spacing=0)
        box.set_border_width(5)
        frame.add(box)

        model = self.create_stock_icon_store()
        combo = Gtk.ComboBox(model=model)
        box.add(combo)

        renderer = Gtk.CellRendererPixbuf()
        combo.pack_start(renderer, False)

        # FIXME: override set_attributes
        combo.add_attribute(renderer, 'pixbuf', PIXBUF_COL)
        combo.set_cell_data_func(renderer, self.set_sensitive, None)

        renderer = Gtk.CellRendererText()
        combo.pack_start(renderer, True)
        combo.add_attribute(renderer, 'text', TEXT_COL)
        combo.set_cell_data_func(renderer, self.set_sensitive, None)

        combo.set_row_separator_func(self.is_separator, None)
        combo.set_active(0)

        # a combobox demonstrating trees
        frame = Gtk.Frame(label='Where are we ?')
        vbox.pack_start(frame, False, False, 0)

        box = Gtk.VBox(homogeneous=False, spacing=0)
        box.set_border_width(5)
        frame.add(box)

        model = self.create_capital_store()
        combo = Gtk.ComboBox(model=model)
        box.add(combo)

        renderer = Gtk.CellRendererText()
        combo.pack_start(renderer, True)
        combo.add_attribute(renderer, 'text', 0)
        combo.set_cell_data_func(renderer, self.is_capital_sensistive, None)

        # FIXME: make new_from_indices work
        #        make constructor take list or string of indices
        path = Gtk.TreePath.new_from_string('0:8')
        treeiter = model.get_iter(path)
        combo.set_active_iter(treeiter)

        # A GtkComboBoxEntry with validation.

        frame = Gtk.Frame(label='Editable')
        vbox.pack_start(frame, False, False, 0)

        box = Gtk.VBox(homogeneous=False, spacing=0)
        box.set_border_width(5)
        frame.add(box)

        combo = Gtk.ComboBoxText.new_with_entry()
        self.fill_combo_entry(combo)
        box.add(combo)

        entry = MaskEntry(mask='^([0-9]*|One|Two|2\302\275|Three)$')

        Gtk.Container.remove(combo, combo.get_child())
        combo.add(entry)

        # A combobox with string IDs

        frame = Gtk.Frame(label='String IDs')
        vbox.pack_start(frame, False, False, 0)

        box = Gtk.VBox(homogeneous=False, spacing=0)
        box.set_border_width(5)
        frame.add(box)

        # FIXME: model is not setup when constructing Gtk.ComboBoxText()
        #        so we call new() - Gtk should fix this to setup the model
        #        in __init__, not in the constructor
        combo = Gtk.ComboBoxText.new()
        combo.append('never', 'Not visible')
        combo.append('when-active', 'Visible when active')
        combo.append('always', 'Always visible')
        box.add(combo)

        entry = Gtk.Entry()

        combo.bind_property('active-id',
                            entry, 'text',
                            GObject.BindingFlags.BIDIRECTIONAL)

        box.add(entry)
        self.window.show_all()

    def strip_underscore(self, s):
        return s.replace('_', '')

    def create_stock_icon_store(self):
        stock_id = (Gtk.STOCK_DIALOG_WARNING,
                    Gtk.STOCK_STOP,
                    Gtk.STOCK_NEW,
                    Gtk.STOCK_CLEAR,
                    None,
                    Gtk.STOCK_OPEN)

        cellview = Gtk.CellView()
        store = Gtk.ListStore(GdkPixbuf.Pixbuf, str)

        for id in stock_id:
            if id is not None:
                pixbuf = cellview.render_icon(id, Gtk.IconSize.BUTTON, None)
                item = Gtk.stock_lookup(id)
                label = self.strip_underscore(item.label)
                store.append((pixbuf, label))
            else:
                store.append((None, 'separator'))

        return store

    def set_sensitive(self, cell_layout, cell, tree_model, treeiter, data):
        """
        A GtkCellLayoutDataFunc that demonstrates how one can control
        sensitivity of rows. This particular function does nothing
        useful and just makes the second row insensitive.
        """

        path = tree_model.get_path(treeiter)
        indices = path.get_indices()

        sensitive = not(indices[0] == 1)

        cell.set_property('sensitive', sensitive)

    def is_separator(self, model, treeiter, data):
        """
        A GtkTreeViewRowSeparatorFunc that demonstrates how rows can be
        rendered as separators. This particular function does nothing
        useful and just turns the fourth row into a separator.
        """

        path = model.get_path(treeiter)

        indices = path.get_indices()
        result = (indices[0] == 4)

        return result

    def create_capital_store(self):
        capitals = (
            {'group': 'A - B', 'capital': None},
            {'group': None, 'capital': 'Albany'},
            {'group': None, 'capital': 'Annapolis'},
            {'group': None, 'capital': 'Atlanta'},
            {'group': None, 'capital': 'Augusta'},
            {'group': None, 'capital': 'Austin'},
            {'group': None, 'capital': 'Baton Rouge'},
            {'group': None, 'capital': 'Bismarck'},
            {'group': None, 'capital': 'Boise'},
            {'group': None, 'capital': 'Boston'},
            {'group': 'C - D', 'capital': None},
            {'group': None, 'capital': 'Carson City'},
            {'group': None, 'capital': 'Charleston'},
            {'group': None, 'capital': 'Cheyeene'},
            {'group': None, 'capital': 'Columbia'},
            {'group': None, 'capital': 'Columbus'},
            {'group': None, 'capital': 'Concord'},
            {'group': None, 'capital': 'Denver'},
            {'group': None, 'capital': 'Des Moines'},
            {'group': None, 'capital': 'Dover'},
            {'group': 'E - J', 'capital': None},
            {'group': None, 'capital': 'Frankfort'},
            {'group': None, 'capital': 'Harrisburg'},
            {'group': None, 'capital': 'Hartford'},
            {'group': None, 'capital': 'Helena'},
            {'group': None, 'capital': 'Honolulu'},
            {'group': None, 'capital': 'Indianapolis'},
            {'group': None, 'capital': 'Jackson'},
            {'group': None, 'capital': 'Jefferson City'},
            {'group': None, 'capital': 'Juneau'},
            {'group': 'K - O', 'capital': None},
            {'group': None, 'capital': 'Lansing'},
            {'group': None, 'capital': 'Lincon'},
            {'group': None, 'capital': 'Little Rock'},
            {'group': None, 'capital': 'Madison'},
            {'group': None, 'capital': 'Montgomery'},
            {'group': None, 'capital': 'Montpelier'},
            {'group': None, 'capital': 'Nashville'},
            {'group': None, 'capital': 'Oklahoma City'},
            {'group': None, 'capital': 'Olympia'},
            {'group': 'P - S', 'capital': None},
            {'group': None, 'capital': 'Phoenix'},
            {'group': None, 'capital': 'Pierre'},
            {'group': None, 'capital': 'Providence'},
            {'group': None, 'capital': 'Raleigh'},
            {'group': None, 'capital': 'Richmond'},
            {'group': None, 'capital': 'Sacramento'},
            {'group': None, 'capital': 'Salem'},
            {'group': None, 'capital': 'Salt Lake City'},
            {'group': None, 'capital': 'Santa Fe'},
            {'group': None, 'capital': 'Springfield'},
            {'group': None, 'capital': 'St. Paul'},
            {'group': 'T - Z', 'capital': None},
            {'group': None, 'capital': 'Tallahassee'},
            {'group': None, 'capital': 'Topeka'},
            {'group': None, 'capital': 'Trenton'}
        )

        parent = None

        store = Gtk.TreeStore(str)

        for item in capitals:
            if item['group']:
                parent = store.append(None, (item['group'],))
            elif item['capital']:
                store.append(parent, (item['capital'],))

        return store

    def is_capital_sensistive(self, cell_layout, cell, tree_model, treeiter, data):
        sensitive = not tree_model.iter_has_child(treeiter)
        cell.set_property('sensitive', sensitive)

    def fill_combo_entry(self, entry):
        entry.append_text('One')
        entry.append_text('Two')
        entry.append_text('2\302\275')
        entry.append_text('Three')


def main(demoapp=None):
    ComboboxApp(demoapp)
    Gtk.main()


if __name__ == '__main__':
    main()



# Copyright 2015 Dustin Spicuzza <dustin@virtualroadside.com>
#           2018 Nikita Churaev <lamefun.x0r@gmail.com>
#           2018 Christoph Reiter <reiter.christoph@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

from gi.repository import GLib, GObject, Gio


def connect_func(builder, obj, signal_name, handler_name,
                 connect_object, flags, cls):

    if handler_name not in cls.__gtktemplate_methods__:
        return

    method_name = cls.__gtktemplate_methods__[handler_name]
    template_inst = builder.get_object(cls.__gtype_name__)
    template_inst.__gtktemplate_handlers__.add(handler_name)
    handler = getattr(template_inst, method_name)

    after = int(flags & GObject.ConnectFlags.AFTER)
    swapped = int(flags & GObject.ConnectFlags.SWAPPED)
    if swapped:
        raise RuntimeError(
            "%r not supported" % GObject.ConnectFlags.SWAPPED)

    if connect_object is not None:
        if after:
            func = obj.connect_object_after
        else:
            func = obj.connect_object
        func(signal_name, handler, connect_object)
    else:
        if after:
            func = obj.connect_after
        else:
            func = obj.connect
        func(signal_name, handler)


def register_template(cls):
    bound_methods = {}
    bound_widgets = {}

    for attr_name, obj in list(cls.__dict__.items()):
        if isinstance(obj, CallThing):
            setattr(cls, attr_name, obj._func)
            handler_name = obj._name
            if handler_name is None:
                handler_name = attr_name

            if handler_name in bound_methods:
                old_attr_name = bound_methods[handler_name]
                raise RuntimeError(
                    "Error while exposing handler %r as %r, "
                    "already available as %r" % (
                        handler_name, attr_name, old_attr_name))
            else:
                bound_methods[handler_name] = attr_name
        elif isinstance(obj, Child):
            widget_name = obj._name
            if widget_name is None:
                widget_name = attr_name

            if widget_name in bound_widgets:
                old_attr_name = bound_widgets[widget_name]
                raise RuntimeError(
                    "Error while exposing child %r as %r, "
                    "already available as %r" % (
                        widget_name, attr_name, old_attr_name))
            else:
                bound_widgets[widget_name] = attr_name
                cls.bind_template_child_full(widget_name, obj._internal, 0)

    cls.__gtktemplate_methods__ = bound_methods
    cls.__gtktemplate_widgets__ = bound_widgets

    cls.set_connect_func(connect_func, cls)

    base_init_template = cls.init_template
    cls.__dontuse_ginstance_init__ = \
        lambda s: init_template(s, cls, base_init_template)
    # To make this file work with older PyGObject we expose our init code
    # as init_template() but make it a noop when we call it ourselves first
    cls.init_template = cls.__dontuse_ginstance_init__


def init_template(self, cls, base_init_template):
    self.init_template = lambda s: None

    if self.__class__ is not cls:
        raise TypeError(
            "Inheritance from classes with @Gtk.Template decorators "
            "is not allowed at this time")

    self.__gtktemplate_handlers__ = set()

    base_init_template(self)

    for widget_name, attr_name in self.__gtktemplate_widgets__.items():
        self.__dict__[attr_name] = self.get_template_child(cls, widget_name)

    for handler_name, attr_name in self.__gtktemplate_methods__.items():
        if handler_name not in self.__gtktemplate_handlers__:
            raise RuntimeError(
                "Handler '%s' was declared with @Gtk.Template.Callback "
                "but was not present in template" % handler_name)


class Child(object):

    def __init__(self, name=None, **kwargs):
        self._name = name
        self._internal = kwargs.pop("internal", False)
        if kwargs:
            raise TypeError("Unhandled arguments: %r" % kwargs)


class CallThing(object):

    def __init__(self, name, func):
        self._name = name
        self._func = func


class Callback(object):

    def __init__(self, name=None):
        self._name = name

    def __call__(self, func):
        return CallThing(self._name, func)


def validate_resource_path(path):
    """Raises GLib.Error in case the resource doesn't exist"""

    try:
        Gio.resources_get_info(path, Gio.ResourceLookupFlags.NONE)
    except GLib.Error:
        # resources_get_info() doesn't handle overlays but we keep using it
        # as a fast path.
        # https://gitlab.gnome.org/GNOME/pygobject/issues/230
        Gio.resources_lookup_data(path, Gio.ResourceLookupFlags.NONE)


class Template(object):

    def __init__(self, **kwargs):
        self.string = None
        self.filename = None
        self.resource_path = None
        if "string" in kwargs:
            self.string = kwargs.pop("string")
        elif "filename" in kwargs:
            self.filename = kwargs.pop("filename")
        elif "resource_path" in kwargs:
            self.resource_path = kwargs.pop("resource_path")
        else:
            raise TypeError(
                "Requires one of the following arguments: "
                "string, filename, resource_path")

        if kwargs:
            raise TypeError("Unhandled keyword arguments %r" % kwargs)

    @classmethod
    def from_file(cls, filename):
        return cls(filename=filename)

    @classmethod
    def from_string(cls, string):
        return cls(string=string)

    @classmethod
    def from_resource(cls, resource_path):
        return cls(resource_path=resource_path)

    Callback = Callback

    Child = Child

    def __call__(self, cls):
        from gi.repository import Gtk

        if not isinstance(cls, type) or not issubclass(cls, Gtk.Widget):
            raise TypeError("Can only use @Gtk.Template on Widgets")

        if "__gtype_name__" not in cls.__dict__:
            raise TypeError(
                "%r does not have a __gtype_name__. Set it to the name "
                "of the class in your template" % cls.__name__)

        if hasattr(cls, "__gtktemplate_methods__"):
            raise TypeError("Cannot nest template classes")

        if self.string is not None:
            data = self.string
            if not isinstance(data, bytes):
                data = data.encode("utf-8")
            bytes_ = GLib.Bytes.new(data)
            cls.set_template(bytes_)
            register_template(cls)
            return cls
        elif self.resource_path is not None:
            validate_resource_path(self.resource_path)
            cls.set_template_from_resource(self.resource_path)
            register_template(cls)
            return cls
        else:
            assert self.filename is not None
            file_ = Gio.File.new_for_path(self.filename)
            bytes_ = GLib.Bytes.new(file_.load_contents()[1])
            cls.set_template(bytes_)
            register_template(cls)
            return cls


__all__ = ["Template"]
